// QFeynCADEditor.C
// Copyright (C) 1999-2004 Paolo Ribeca

// This file is part of FeynCAD package.

// FeynCAD is free software;
// you can redistribute it and/or modify it under the terms
// of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with FeynCAD; see the file COPYING.
// If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#define FEYNCAD_DEFAULT_GRID_X 40
#define FEYNCAD_DEFAULT_GRID_Y 40
#define FEYNCAD_DEFAULT_GRID_STEP 10
#define FEYNCAD_DEFAULT_SQUARE_BASE_SIZE 1
#define FEYNCAD_DEFAULT_CIRCLE_BASE_SIZE 4
#define FEYNCAD_DEFAULT_MARKER_STEP 5
#define FEYNCAD_DEFAULT_MACROS_PATH "FeynCAD.mp"
#define FEYNCAD_DEFAULT_ADDENDUM_PATH ""
#define FEYNCAD_DEFAULT_METAPOST_PATH "mpost"
#define FEYNCAD_DEFAULT_LATEX_PATH "latex"
#define FEYNCAD_DEFAULT_DVIPS_PATH "dvips"
#define FEYNCAD_DEFAULT_GS_PATH "gs"
#define FEYNCAD_DEFAULT_BROWSER_PATH ""
#define FEYNCAD_DEFAULT_TMP_PATH "/tmp"
#define FEYNCAD_DEFAULT_TIMEOUT 5
#define FEYNCAD_DEFAULT_KEEP_FILES false
#define FEYNCAD_DEFAULT_HISTORY_DEPTH 100
#define FEYNCAD_DEFAULT_METAPOST_SETTINGS "2 1.5 0 2.5 0 3 0 3 0 2.4 0 4 0 4 0 2.4 15 160 60"

#include <cmath>
#include <sstream>
#include <fstream>
#include <map>
#include <limits>
#include <iomanip>

#include <qapplication.h>
#include <qpainter.h>
#include <qbrush.h>
#include <qmessagebox.h>
#include <qpointarray.h>
#include <qgroupbox.h>
#include <qvbox.h>
#include <qcursor.h>
#include <qfont.h>
#include <qdom.h>
#include <qxml.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qdatetime.h>
#include <qpushbutton.h>

using namespace std;

float metaFONTVelocity(const float& theta,const float& phi,const float& tau) {
  static float sqrtOf2=sqrt(float(2)),sqrtOf5=sqrt(float(5));
  float st=sin(theta),ct=cos(theta),sp=sin(phi),cp=cos(phi);
  float res=(float(2)+sqrtOf2*(st-sp/float(16))*(sp-st/float(16))*(ct-cp))
    /tau/float(3)
    /(float(1)+(sqrtOf5-float(1))/float(2)*ct+(float(3)-sqrtOf5)/float(2)*cp);
  return (res>float(4)?float(4):res);
}
void metaFONTControlPoints(QPointArray& arr,const float& dirBeg,
			   const float& dirEnd,const float& tau) {
  if (arr[0]==arr[3])
    { arr[1]=arr[0]; arr[2]=arr[3]; return; }
  // _Remember_the_infamous_Y-axis_sign_of_Qt's_frame_
  float dist=hypot(arr[3].x()-arr[0].x(),arr[0].y()-arr[3].y());
  float arg=atan2(float(arr[0].y()-arr[3].y()),float(arr[3].x()-arr[0].x()));
  if (dirBeg==-361&&dirEnd==-361) { // Case of the straight line
    arr[1]=QPoint(int(arr[0].x()+dist*cos(arg)/float(3)),
		  int(arr[0].y()-dist*sin(arg)/float(3)));
    arr[2]=QPoint(int(arr[3].x()-dist*cos(arg)/float(3)),
		  int(arr[3].y()+dist*sin(arg)/float(3)));
    return;
  }
  float theta=dirBeg,phi=dirEnd;
  // A questo punto almeno uno dei due angoli e' dato
  // Per rispettare le convenzioni:
  //  -0- coordinate polari
  //  -1- gli angoli devono diventare relativi
  //  -2- phi deve essere riflesso
  if (dirBeg!=-361)
    { theta*=(M_PI/180); theta-=arg; }
  if (dirEnd!=-361)
    { phi*=(M_PI/180); phi=arg-phi; }
  // Se uno solo degli angoli e' dato si stabilisce anche l'altro
  if (dirBeg==-361)
    theta=phi;
  else if (dirEnd==-361)
    phi=theta;
  // Forse qui il default dovrebbe essere 1... il punto e' che non so
  //  esattamente come si comportano MetaFONT/MetaPost in questo caso
  float effTau=(tau<float(3)/float(4)?float(3)/float(4):tau);
  float velBeg=metaFONTVelocity(theta,phi,effTau);
  arr[1]=QPoint(int(arr[0].x()+velBeg*dist*cos(theta+arg)),
		int(arr[0].y()-velBeg*dist*sin(theta+arg)));
  float velEnd=metaFONTVelocity(phi,theta,effTau);
  arr[2]=QPoint(int(arr[3].x()-velEnd*dist*cos(arg-phi)),
		int(arr[3].y()+velEnd*dist*sin(arg-phi)));
  return;
}
inline unsigned measure(int num) {
  unsigned res=0;
  if (num<0)
    ++res;
  num=(num<0?-num:num);
  if (num<10) ++res;
  else if (num<100) res+=2;
  else if (num<1000) res+=3;
  else if (num<10000) res+=4;
  else res+=unsigned(floor(log10(double(num))));
  return res;
}

#include "QFeynCADEditor.h"

QFeynCADEditor::QFeynCADEditor(QWidget* parent,const char* name,
			       int gridSizeX,int gridSizeY,string fileName):
  QWidget(parent,name),
  _mode(QFeynCADEditor::DrawMode), _last(QFeynCADEditor::Nothing),
  _tracingLine(false),
  _selRectOne(make_pair(-1,-1)), _selRectTwo(make_pair(-1,-1)),
  _newSelRectOne(make_pair(-1,-1)), _newSelRectTwo(make_pair(-1,-1)),
  _newSelRectRotationCenter(make_pair(-1,-1)), _newSelRectRotationAngle(0),
  _current(0), _fileName(fileName), _inBatchMode(false)
{
  this->setPaletteBackgroundColor(white);
  this->_status.gridSize=make_pair(gridSizeX,gridSizeY);
  this->_isUpToDate=true; // Tutti e due sono necessari
  this->pushHistory("Base");
  this->_isUpToDate=true; // Tutti e due sono necessari
  emit titleUpdateReq();
  emit gridChanged(this->_status.gridSize.first,this->_status.gridSize.second);
  this->setFixedSize(this->_status.gridSize.first*QFeynCADEditor::_gridStep+
		     2*QFeynCADEditor::_circleBaseSize+1,
		     this->_status.gridSize.second*QFeynCADEditor::_gridStep+
		     2*QFeynCADEditor::_circleBaseSize+1);
  this->_modifyOrDelete=new QPopupMenu(this);
  connect(this->_modifyOrDelete,SIGNAL(aboutToShow()),
	  this,SLOT(updateModifyOrDelete()));
  connect(this->_modifyOrDelete,SIGNAL(aboutToHide()),
	  this,SLOT(mergeAlphaUniquifying()));
  this->_runawayDialogCatcher=new QTimer(this);
  connect(this->_runawayDialogCatcher,SIGNAL(timeout()),
	  this,SLOT(reenableApp()));
  this->_modifyVertex=new QFeynCADVertexEdit(0);
  connect(this->_modifyVertex->PushButtonApply,SIGNAL(clicked()),
	  this,SLOT(modifyVertex()));
  connect(this->_modifyVertex->PushButtonClose,SIGNAL(clicked()),
	  this,SLOT(mergeAlphaUniquifying()));
  connect(this->_modifyVertex->PushButtonClose,SIGNAL(clicked()),
	  this,SLOT(updateVertexList()));
  connect(this->_modifyVertex->PushButtonClose,SIGNAL(clicked()),
	  this,SLOT(stopRunawayDialogCatcher()));
  // Per (reenableApp)
  this->_modifyLineParent=0;
  this->_modifyTag=new QFeynCADTagEdit(0);
  connect(this->_modifyTag->PushButtonApply,SIGNAL(clicked()),
	  this,SLOT(modifyTag()));
  connect(this->_modifyTag->PushButtonClose,SIGNAL(clicked()),
	  this,SLOT(mergeAlphaUniquifying()));
  connect(this->_modifyTag->PushButtonClose,SIGNAL(clicked()),
	  this,SLOT(updateTagList()));
  connect(this->_modifyTag->PushButtonClose,SIGNAL(clicked()),
	  this,SLOT(stopRunawayDialogCatcher()));
  this->_modifySelection=new QFeynCADTransformEdit(0);
  // In questi casi lo stop al runaway catcher e' incluso nelle chiamate
  connect(this->_modifySelection->PushButtonClose,SIGNAL(clicked()),
	  this,SLOT(abandonTransformation()));
  connect(this->_modifySelection,SIGNAL(clipToSelectionReq()),
	  this,SLOT(clipToSelection()));
  connect(this->_modifySelection,SIGNAL(rotateAlphaReq(int,int,int)),
	  this,SLOT(rotateAlpha(int,int,int)));
  connect(this->_modifySelection,SIGNAL(acceptRotationReq()),
	  this,SLOT(acceptRotation()));
  this->setMouseTracking(true);
  // Serve solo nel (PasteMode)
  this->_accelerator=new QAccel(this);
  this->_accelerator->connectItem(this->_accelerator->insertItem(Key_Enter),
				  this,SLOT(acceptPaste()));
  this->_accelerator->connectItem(this->_accelerator->insertItem(Key_Return),
				  this,SLOT(acceptPaste()));
  this->_accelerator->connectItem(this->_accelerator->insertItem(Key_Escape),
				  this,SLOT(abandonPaste()));
  this->_accelerator->setEnabled(false);
  // Questo e' _fondamentale_ per evitare il flickering
  this->setBackgroundMode(Qt::NoBackground);
  this->repaint();
}
QFeynCADEditor::~QFeynCADEditor() {
  ostringstream removeCommand;
  removeCommand << "rm -f " << this->_fileName << ".eps ";
  if (!QFeynCADEditor::_keepFiles)
    removeCommand << this->_fileName << ".mp " << this->_fileName << ".tex ";
  removeCommand << ">& /dev/null 2>&1";
  if (system(removeCommand.str().c_str())) {
    if (!this->_inBatchMode) {
      ostringstream message;
      message << "Could not properly remove temporary files\n(\""
	      << removeCommand.str() << "\").";
      QMessageBox::warning(this,"QFeynCADEditor",message.str().c_str());
    }
  }
}

void QFeynCADEditor::undoRedoUpdate() {
  // (_current) in realta' e' <= (_history.size())
  string first=(this->_current>1?this->_history[this->_current-1].second:""),
    second=(this->_current<this->_history.size()?
	    this->_history[this->_current].second:"");
  if (second=="Base")
    abort(); // Agonizing pain
  emit undoRedoUpdateReq(first,second);
  return;
}
void QFeynCADEditor::pushHistory(string description) {
  // Prima di tutto si controlla che lo stato corrente sia diverso
  //  dall'entry corrente nella history
  if (this->_current&&
      this->_history[this->_current-1].first.gridSize==this->_status.gridSize&&
      this->_history[this->_current-1].first.vertices==this->_status.vertices&&
      this->_history[this->_current-1].first.lines==this->_status.lines&&
      this->_history[this->_current-1].first.tags==this->_status.tags)
    return;
  if (this->_current!=this->_history.size()) {
    // Siamo alla fine di una serie di undo/redo
    //  Si cancella l'history dallo stato corrente in poi
    deque<pair<status,string> >::iterator it=
      this->_history.begin()+this->_current;
    this->_history.erase(it,this->_history.end());
  }
  //  Si inserisce nella history lo stato corrente
  this->_history.push_back(make_pair(this->_status,description));
  //   Se il size attuale e' oltre la depth massima la storia viene accorciata
  while (this->_history.size()>QFeynCADEditor::_historyDepth)
    this->_history.pop_front();
  this->_current=this->_history.size();
  emit message(false,string("Done \"")+description+string("\""));
  if (this->_isUpToDate)
    { this->_isUpToDate=false; emit titleUpdateReq(); }
  this->undoRedoUpdate();
  // Aggiornamento dei combo-box
  this->updateVertexList();
  this->updateLineList();
  this->updateTagList();
  return;
}
void QFeynCADEditor::goBackInHistory() {
  if (this->_current<2||this->_history[this->_current-1].second=="Base")
    return;
  --this->_current;
  if (this->_status.gridSize!=this->_history[this->_current-1].first.gridSize)
    this->setGridSizeTo(this->_history[this->_current-1].first.gridSize.first,
			this->_history[this->_current-1].first.gridSize.
			second);
  this->_status=this->_history[this->_current-1].first;
  this->_alpha.clear();
  if (this->_isUpToDate)
    { this->_isUpToDate=false; emit titleUpdateReq(); }
  this->undoRedoUpdate();
  // Aggiornamento dei combo-box
  this->updateVertexList();
  this->updateLineList();
  this->updateTagList();
  // Disselezione nel (SelectMode)
  if (this->_mode==SelectMode) {
    this->_memoryOne=make_pair(-1,-1);
    this->_memoryTwo=make_pair(-1,-1);
    this->_selRectOne=make_pair(-1,-1);
    this->_selRectTwo=make_pair(-1,-1);
    this->_newSelRectOne=make_pair(-1,-1);
    this->_newSelRectTwo=make_pair(-1,-1);
    emit newSelection();
  }
  //
  this->repaint();
  this->updatePostscript();
  emit message(false,string("Undone \"")+
	       this->_history[this->_current].second+string("\""));
  return;
}
void QFeynCADEditor::goForwardInHistory() {
  if (this->_current==this->_history.size())
    return;
  ++this->_current;
  if (this->_status.gridSize!=this->_history[this->_current-1].first.gridSize)
    this->setGridSizeTo(this->_history[this->_current-1].first.gridSize.first,
			this->_history[this->_current-1].first.gridSize.
			second);
  this->_status=this->_history[this->_current-1].first;
  this->_alpha.clear();
  if (this->_isUpToDate)
    { this->_isUpToDate=false; emit titleUpdateReq(); }
  this->undoRedoUpdate();
  // Aggiornamento dei combo-box
  this->updateVertexList();
  this->updateLineList();
  this->updateTagList();
  // Disselezione nel (SelectMode)
  if (this->_mode==SelectMode) {
    this->_memoryOne=make_pair(-1,-1);
    this->_memoryTwo=make_pair(-1,-1);
    this->_selRectOne=make_pair(-1,-1);
    this->_selRectTwo=make_pair(-1,-1);
    this->_newSelRectOne=make_pair(-1,-1);
    this->_newSelRectTwo=make_pair(-1,-1);
    emit newSelection();
  }
  //
  this->repaint();
  this->updatePostscript();
  emit message(false,string("Redone \"")+
	       this->_history[this->_current-1].second+string("\""));
  return;
}

int QFeynCADEditor::xGridPosWhenXCanvasIs(int xCanvasPos) const {
  int res=(int)rint((xCanvasPos-QFeynCADEditor::_circleBaseSize)/
		    (double)QFeynCADEditor::_gridStep);
  if (res<0)
    return 0;
  if (res>this->_status.gridSize.first)
    return this->_status.gridSize.first;
  return res;
}
int QFeynCADEditor::yGridPosWhenYCanvasIs(int yCanvasPos) const {
  // Si riflette la coordinata Y
  int res=(int)(this->_status.gridSize.second-
		rint((yCanvasPos-QFeynCADEditor::_circleBaseSize)/
		     (double)QFeynCADEditor::_gridStep));
  if (res<0)
    return 0;
  if (res>this->_status.gridSize.second)
    return this->_status.gridSize.second;
  return res;
}
int QFeynCADEditor::xCanvasPosWhenXGridIs(int xGridPos) const
{ return xGridPos*QFeynCADEditor::_gridStep+QFeynCADEditor::_circleBaseSize; }
int QFeynCADEditor::yCanvasPosWhenYGridIs(int yGridPos) const {
  // Si riflette la coordinata Y
  return (this->_status.gridSize.second-yGridPos)*QFeynCADEditor::_gridStep+
    QFeynCADEditor::_circleBaseSize;
}
QRect QFeynCADEditor::getEnvelope(const pair<int,int>& one,
				  const pair<int,int>& two) const {
  pair<int,int> tl=
    make_pair(min(one.first,two.first),max(one.second,two.second));
  pair<int,int> br=
    make_pair(max(one.first,two.first),min(one.second,two.second));
  // Per sicurezza (vs. selezioni) si sta piu' larghi di un punto della griglia
  --tl.first,++tl.second,++br.first,--br.second;
  // We clip to valid coordinates
  if (tl.first<0)
    tl.first=0;
  if (tl.second>this->_status.gridSize.second)
    tl.second=this->_status.gridSize.second;
  if (br.first>this->_status.gridSize.first)
    br.first=this->_status.gridSize.first;
  if (br.second<0)
    br.second=0;
  return QRect(QPoint(this->xCanvasPosWhenXGridIs(tl.first),
		      this->yCanvasPosWhenYGridIs(tl.second)),
	       QPoint(this->xCanvasPosWhenXGridIs(br.first),
		      this->yCanvasPosWhenYGridIs(br.second)));
}

#define ENABLED_REFERENCE_COLOR QColor(255,165,0) // "Orange"
#define DISABLED_REFERENCE_COLOR QColor(255,215,0) // "Gold"
//
#define SELECTION_RECTANGLE_COLOR gray
//
#define ENABLED_GRID_COLOR black
#define DISABLED_GRID_COLOR darkGray
//
#define ENABLED_TAG_POINT_COLOR darkMagenta
#define ENABLED_TAG_SQUARE_COLOR magenta
#define DISABLED_TAG_POINT_COLOR QColor(224,102,255) // "MediumOrchid1"
#define DISABLED_TAG_SQUARE_COLOR QColor(255,131,250) // "Orchid1"
#define SELECTED_TAG_POINT_COLOR darkRed
#define SELECTED_TAG_SQUARE_COLOR red
//
#define ENABLED_LINE_COLOR blue
#define DISABLED_LINE_COLOR QColor(132,112,255) // "LightSlateBlue"
#define SELECTED_LINE_COLOR red
#define ENABLED_ARROW_COLOR QColor(0,170,0)
#define DISABLED_ARROW_COLOR QColor(104,170,104)
#define SELECTED_ARROW_COLOR QColor(170,0,0)
//
#define ENABLED_VERTEX_COLOR green
#define DISABLED_VERTEX_COLOR QColor(154,255,154)// "PaleGreen1"
#define SELECTED_VERTEX_COLOR red
//
#define ROTATION_CENTER_POINT_COLOR darkCyan
#define ROTATION_CENTER_LINES_COLOR cyan

#define SMOOTH_PEN(color,size) QPen(color,size,SolidLine,RoundCap,RoundJoin)
void QFeynCADEditor::paintEvent(QPaintEvent* e) {
  static QPoint oldStart=QPoint(0,0),oldEnd=QPoint(0,0);
  QRect rec=e->rect();
  //cout << "[" << rec.left() << "-" << rec.right() << "]x[" << rec.top()
  //     << "-" << rec.bottom() << "]" << endl;
  // Nel ridisegnare si sta un po' piu' larghi
  QPoint start(((rec.left()-QFeynCADEditor::_circleBaseSize)/
		QFeynCADEditor::_gridStep)*
	       QFeynCADEditor::_gridStep+QFeynCADEditor::_circleBaseSize,
	       ((rec.top()-QFeynCADEditor::_circleBaseSize)/
		QFeynCADEditor::_gridStep)*
	       QFeynCADEditor::_gridStep+QFeynCADEditor::_circleBaseSize);
  QPoint end(((rec.right()-QFeynCADEditor::_circleBaseSize)/
	      QFeynCADEditor::_gridStep+1)*
	     QFeynCADEditor::_gridStep+QFeynCADEditor::_circleBaseSize,
	     ((rec.bottom()-QFeynCADEditor::_circleBaseSize)/
	      QFeynCADEditor::_gridStep+1)*
	     QFeynCADEditor::_gridStep+QFeynCADEditor::_circleBaseSize);
  // Si disegna nell'area piu' grande tra il vecchio rettangolo e il nuovo
  start=QPoint(min(start.x(),oldStart.x()),min(start.y(),oldStart.y()));
  end=QPoint(max(end.x(),oldEnd.x()),max(end.y(),oldEnd.y()));
  oldStart=start;
  oldEnd=end;
  // Pixmap for flicker-free painting
  QPixmap pixmap(QSize(end.x()-start.x(),end.y()-start.y()));
  pixmap.fill(this,start);
  QPainter p(&pixmap);
  // Si disegnano le linee di riferimento della griglia
  int sX=xGridPosWhenXCanvasIs(start.x()),eX=xGridPosWhenXCanvasIs(end.x());
  int sY=yGridPosWhenYCanvasIs(end.y()),eY=yGridPosWhenYCanvasIs(start.y());
  if (QFeynCADEditor::_markerStep) {
    QColor referenceColor=(this->isEnabled()&&this->_mode==DrawMode?
			   ENABLED_REFERENCE_COLOR:DISABLED_REFERENCE_COLOR);
    QPen pen(referenceColor,2*QFeynCADEditor::_squareBaseSize+1);
    p.setBrush(black);
    p.setPen(pen);
    for (register int i=sX-1;i<eX+1;++i)
      if (!(i%QFeynCADEditor::_markerStep)||i==this->_status.gridSize.first)
	p.drawLine(xCanvasPosWhenXGridIs(i),yCanvasPosWhenYGridIs(sY),
		   xCanvasPosWhenXGridIs(i),yCanvasPosWhenYGridIs(eY));
    for (register int i=sY-1;i<eY+1;++i)
      if (!(i%QFeynCADEditor::_markerStep)||i==this->_status.gridSize.second)
	p.drawLine(xCanvasPosWhenXGridIs(sX),yCanvasPosWhenYGridIs(i),
		   xCanvasPosWhenXGridIs(eX),yCanvasPosWhenYGridIs(i));
  }
  p.setBrush(black);
  p.setPen(NoPen);
  // Si disegna se e' il caso il rettangolo di selezione
  if (this->_selRectTwo!=pair<int,int>(-1,-1)) {
    QBrush brush(SELECTION_RECTANGLE_COLOR);
    brush.setStyle(Qt::Dense4Pattern);
    p.setBrush(brush);
    p.drawRect(min(xCanvasPosWhenXGridIs(this->_selRectOne.first),
		   xCanvasPosWhenXGridIs(this->_selRectTwo.first))-
	       (QFeynCADEditor::_gridStep/2),
	       min(yCanvasPosWhenYGridIs(this->_selRectOne.second),
		   yCanvasPosWhenYGridIs(this->_selRectTwo.second))-
	       (QFeynCADEditor::_gridStep/2),
	       abs(this->_selRectTwo.first-this->_selRectOne.first)*
	       QFeynCADEditor::_gridStep+2*(QFeynCADEditor::_gridStep/2),
	       abs(this->_selRectTwo.second-this->_selRectOne.second)*
	       QFeynCADEditor::_gridStep+2*(QFeynCADEditor::_gridStep/2));
    p.setBrush(black);
    p.setPen(NoPen);
  }
  // Si disegnano le tag [sotto la griglia]
#define DRAW_TAG_1(TAG,rubout) {\
    int width=2*(QFeynCADEditor::_gridStep+rubout),\
      height=2*(QFeynCADEditor::_gridStep+rubout),startPosX=0,startPosY=0;\
    if (TAG->controlPoint=="tl") startPosX=0,startPosY=0;\
    else if (TAG->controlPoint=="tc") startPosX=-width/2,startPosY=0;\
    else if (TAG->controlPoint=="tr") startPosX=-width,startPosY=0;\
    else if (TAG->controlPoint=="cl") startPosX=0,startPosY=-height/2;\
    else if (TAG->controlPoint=="cr") startPosX=-width,startPosY=-height/2;\
    else if (TAG->controlPoint=="bl") startPosX=0,startPosY=-height;\
    else if (TAG->controlPoint=="bc") startPosX=-width/2,startPosY=-height;\
    else if (TAG->controlPoint=="br") startPosX=-width,startPosY=-height;\
    /* Il caso "cc" serve da default */\
    else startPosX=-width/2,startPosY=-height/2;\
    p.save();\
    /* L'(1) e' la meta' del bordo */\
    p.translate(xCanvasPosWhenXGridIs(TAG->pos.first)+1+rubout,\
		yCanvasPosWhenYGridIs(TAG->pos.second)+1+rubout);\
    p.rotate(-TAG->startingAngle);\
    p.setPen(SMOOTH_PEN(tagSquareColor,2));\
    p.setBrush(tagSquareColor);\
    p.drawRoundRect(startPosX,startPosY,width,height);\
    /*p.setFont(QFont("Times",25*QFeynCADEditor::_gridStep/10,QFont::Bold));\
    p.drawText(startPosX,startPosY,width,height,\
               Qt::AlignCenter|Qt::SingleLine|Qt::DontClip,\
               TAG->text.c_str());*/\
    p.restore();\
  }
#define DRAW_TAG_2(TAG) {\
    p.setPen(tagPointColor);\
    p.setBrush(tagPointColor);\
    p.drawEllipse(xCanvasPosWhenXGridIs(TAG->pos.first)-\
		  3*QFeynCADEditor::_circleBaseSize/2,\
		  yCanvasPosWhenYGridIs(TAG->pos.second)-\
		  3*QFeynCADEditor::_circleBaseSize/2,\
		  2*(3*QFeynCADEditor::_circleBaseSize/2)+1,\
		  2*(3*QFeynCADEditor::_circleBaseSize/2)+1);\
  }
  // Qui si disegnano tutti i punti sopra
  for (set<tag>::const_iterator
	 i=this->_status.tags.begin();i!=this->_status.tags.end();++i)
    if (i->pos.first>=0||i->pos.second>=0) {
      QColor tagSquareColor=white;
      DRAW_TAG_1(i,1);
      tagSquareColor=(this->isEnabled()?
		      ENABLED_TAG_SQUARE_COLOR:DISABLED_TAG_SQUARE_COLOR);
      DRAW_TAG_1(i,0);
    }
  for (set<tag>::const_iterator
	 i=this->_status.tags.begin();i!=this->_status.tags.end();++i)
    if (i->pos.first>=0||i->pos.second>=0) {
      QColor tagPointColor=(this->isEnabled()?
			    ENABLED_TAG_POINT_COLOR:DISABLED_TAG_POINT_COLOR);
      DRAW_TAG_2(i);
    }
  for (set<tag>::const_iterator
	 i=this->_alpha.tags.begin();i!=this->_alpha.tags.end();++i)
    if (i->pos.first>=0||i->pos.second>=0) {
      QColor tagSquareColor=white;
      DRAW_TAG_1(i,1);
      tagSquareColor=SELECTED_TAG_SQUARE_COLOR;
      DRAW_TAG_1(i,0);
    }
  for (set<tag>::const_iterator
	 i=this->_alpha.tags.begin();i!=this->_alpha.tags.end();++i)
    if (i->pos.first>=0||i->pos.second>=0) {
      QColor tagPointColor=SELECTED_TAG_POINT_COLOR;
      DRAW_TAG_2(i);
    }
#undef DRAW_TAG_1
#undef DRAW_TAG_2
  // Si disegna la griglia
  QColor gridColor=(this->isEnabled()&&this->_mode==DrawMode?
		    ENABLED_GRID_COLOR:DISABLED_GRID_COLOR);
  p.setBrush(gridColor);
  p.setPen(NoPen);
  for (register int i=sX-1;i<eX+1;++i)
    for (register int j=sY-1;j<eY+1;++j)
      p.drawRect(xCanvasPosWhenXGridIs(i)-QFeynCADEditor::_squareBaseSize,
		 yCanvasPosWhenYGridIs(j)-QFeynCADEditor::_squareBaseSize,
		 2*QFeynCADEditor::_squareBaseSize+1,
		 2*QFeynCADEditor::_squareBaseSize+1);
  // Si disegnano le linee [come delle spline di MetaPost]
#define DRAW_ARROW(border) {\
  dir.first=dir.first/hyp*QFeynCADEditor::_gridStep*1.6;\
  dir.second=dir.second/hyp*QFeynCADEditor::_gridStep*1.6;\
  /*dirL=QPoint(-dir.y(),dir.x());*/\
  /*dirR=QPoint(dir.y(),-dir.x());*/\
  arrowArr[0]=\
    QPoint(int(round(middle.first+dir.first*float(2)/float(3))),\
           int(round(middle.second+dir.second*float(2)/float(3))));\
  arrowArr[1]=QPoint(int(round(middle.first-dir.first/float(3)\
			       -dir.second*(float(4)/float(15)))),\
		     int(round(middle.second-dir.second/float(3)\
			       +dir.first*(float(4)/float(15)))));\
  arrowArr[2]=QPoint(int(round(middle.first-dir.first/float(3)\
			       +dir.second*(float(4)/float(15)))),\
		     int(round(middle.second-dir.second/float(3)\
			       -dir.first*(float(4)/float(15)))));\
  p.setPen(SMOOTH_PEN(arrowColor,3+border));\
  p.setBrush(arrowColor);\
  p.drawConvexPolygon(arrowArr);\
}
  //
#define DRAW_LINE(LINE,rubout) {\
  p.setPen(QPen(lineColor,QFeynCADEditor::_circleBaseSize+2*rubout));\
  if (LINE->vOne==LINE->vTwo) {\
    QPointArray arrOne(4),arrTwo(4),arrowArr(3);\
    arrOne[0]=QPoint(xCanvasPosWhenXGridIs(LINE->vOne.first),\
		     yCanvasPosWhenYGridIs(LINE->vOne.second));\
    arrOne[3]=QPoint(int(xCanvasPosWhenXGridIs(LINE->vOne.first)+\
		         LINE->tension*QFeynCADEditor::_gridStep*\
		         cos((LINE->angleOne)*M_PI/float(180))),\
		     int(yCanvasPosWhenYGridIs(LINE->vOne.second)-\
                         LINE->tension*QFeynCADEditor::_gridStep*\
	                 sin((LINE->angleOne)*M_PI/float(180))));\
    arrTwo[0]=arrOne[3];\
    arrTwo[3]=QPoint(xCanvasPosWhenXGridIs(LINE->vTwo.first),\
		     yCanvasPosWhenYGridIs(LINE->vTwo.second));\
    metaFONTControlPoints(arrOne,LINE->angleOne-LINE->angleTwo/float(2),\
                          float(LINE->angleTwo<0?-1:+1)*float(90)+\
                          LINE->angleOne,\
                          float(1));\
    metaFONTControlPoints(arrTwo,float(LINE->angleTwo<0?-1:+1)*float(90)+\
			  LINE->angleOne,\
			  float(180)+LINE->angleOne+LINE->angleTwo/float(2),\
			  float(1));\
    p.drawCubicBezier(arrOne);\
    p.drawCubicBezier(arrTwo);\
    p.save();\
    pair<float,float> middle(arrOne[3].x(),arrOne[3].y());\
    pair<float,float> dir(arrOne[3].x()-arrOne[2].x(),\
                          arrOne[3].y()-arrOne[2].y());\
    float hyp=hypot(dir.first,dir.second);\
    if (!hyp) hyp=float(1);\
    DRAW_ARROW(2*rubout)\
    p.restore();\
  } else {\
    QPointArray arr(4),arrowArr(3);\
    arr[0]=QPoint(xCanvasPosWhenXGridIs(LINE->vOne.first),\
		  yCanvasPosWhenYGridIs(LINE->vOne.second));\
    arr[3]=QPoint(xCanvasPosWhenXGridIs(LINE->vTwo.first),\
		  yCanvasPosWhenYGridIs(LINE->vTwo.second));\
    metaFONTControlPoints(arr,LINE->angleOne,LINE->angleTwo,LINE->tension);\
    p.drawCubicBezier(arr);\
    p.save();\
    pair<float,float> middle(0.125*arr[0].x()+0.375*arr[1].x()+\
		             0.375*arr[2].x()+0.125*arr[3].x(),\
		             0.125*arr[0].y()+0.375*arr[1].y()+\
		             0.375*arr[2].y()+0.125*arr[3].y());\
    pair<float,float> dir(-0.75*arr[0].x()-0.75*arr[1].x()\
		          +0.75*arr[2].x()+0.75*arr[3].x(),\
		          -0.75*arr[0].y()-0.75*arr[1].y()\
		          +0.75*arr[2].y()+0.75*arr[3].y());\
    float hyp=hypot(dir.first,dir.second);\
    DRAW_ARROW(2*rubout)\
    p.restore();\
  }\
}
  //
  for (multiset<line>::const_iterator
	 i=this->_status.lines.begin();i!=this->_status.lines.end();++i)
    if (i->vOne.first>=0||i->vTwo.second>=0||
	i->vTwo.first>=0||i->vTwo.second>=0) {
      QColor lineColor=white,arrowColor=white;
      DRAW_LINE(i,1);
      lineColor=(this->isEnabled()?ENABLED_LINE_COLOR:DISABLED_LINE_COLOR);
      arrowColor=(this->isEnabled()?ENABLED_ARROW_COLOR:DISABLED_ARROW_COLOR);
      DRAW_LINE(i,0);
    }
  for (multiset<line>::const_iterator
	 i=this->_alpha.lines.begin();i!=this->_alpha.lines.end();++i)
    if (i->vOne.first>=0||i->vTwo.second>=0||
	i->vTwo.first>=0||i->vTwo.second>=0) {
      QColor lineColor=white,arrowColor=white;
      DRAW_LINE(i,1);
      lineColor=SELECTED_LINE_COLOR,arrowColor=SELECTED_ARROW_COLOR;
      DRAW_LINE(i,0);
    }
#undef DRAW_ARROW
#undef DRAW_LINE
  p.setPen(NoPen);
  // Si disegna se e' il caso la linea che si sta tracciando
  if (this->_tracingLine) {
    QPen pen(SELECTED_LINE_COLOR,4*QFeynCADEditor::_squareBaseSize);
    pen.setStyle(Qt::DotLine);
    p.setPen(pen);
    p.drawLine(xCanvasPosWhenXGridIs(this->_memoryOne.first),
	       yCanvasPosWhenYGridIs(this->_memoryOne.second),
	       xCanvasPosWhenXGridIs(this->_memoryTwo.first),
	       yCanvasPosWhenYGridIs(this->_memoryTwo.second));
    p.setBrush(black);
    p.setPen(NoPen);
  }
  // Si disegnano i vertici
#define DRAW_VERTEX(VERTEX) {\
    p.setBrush(vertexColor);\
    p.drawEllipse(xCanvasPosWhenXGridIs(VERTEX->pos.first)-\
		  QFeynCADEditor::_circleBaseSize,\
		  yCanvasPosWhenYGridIs(VERTEX->pos.second)-\
		  QFeynCADEditor::_circleBaseSize,\
		  2*QFeynCADEditor::_circleBaseSize+1,\
		  2*QFeynCADEditor::_circleBaseSize+1);\
  }
  //
  for (set<vertex>::const_iterator i=this->_status.vertices.begin();
       i!=this->_status.vertices.end();++i)
    if (i->pos.first>=0||i->pos.second>=0) {
      QColor vertexColor=
	(this->isEnabled()?ENABLED_VERTEX_COLOR:DISABLED_VERTEX_COLOR);
      DRAW_VERTEX(i);
    }
  for (set<vertex>::const_iterator i=this->_alpha.vertices.begin();
       i!=this->_alpha.vertices.end();++i)
    if (i->pos.first>=0||i->pos.second>=0) {
      QColor vertexColor=SELECTED_VERTEX_COLOR;
      DRAW_VERTEX(i);
    }
#undef DRAW_VERTEX
  // Si plotta la cornice di selezione
  bool case1=(this->_newSelRectOne!=pair<int,int>(-1,-1)&&
	      this->_newSelRectTwo!=pair<int,int>(-1,-1));
  bool case2=(this->_newSelRectRotationCenter!=pair<int,int>(-1,-1)&&
	      (this->_selRectOne!=pair<int,int>(-1,-1)&&
	       this->_selRectTwo!=pair<int,int>(-1,-1)));
  if (case1||case2) {
    pair<int,int> pointOne,pointTwo;
    if (case2)
      pointOne=this->_selRectOne,pointTwo=this->_selRectTwo;
    else
      pointOne=this->_newSelRectOne,pointTwo=this->_newSelRectTwo;
    int xBase,yBase,width,height;
    xBase=min(xCanvasPosWhenXGridIs(pointOne.first),
	      xCanvasPosWhenXGridIs(pointTwo.first))-
      (QFeynCADEditor::_gridStep/2);
    yBase=min(yCanvasPosWhenYGridIs(pointOne.second),
	      yCanvasPosWhenYGridIs(pointTwo.second))-
      (QFeynCADEditor::_gridStep/2);
    width=abs(pointTwo.first-pointOne.first)*
      QFeynCADEditor::_gridStep+2*(QFeynCADEditor::_gridStep/2);
    height=abs(pointTwo.second-pointOne.second)*
      QFeynCADEditor::_gridStep+2*(QFeynCADEditor::_gridStep/2);
    if (case2) {
      // Si disegna il centro di rotazione
      int
	xCenter=xCanvasPosWhenXGridIs(this->_newSelRectRotationCenter.first),
	yCenter=yCanvasPosWhenYGridIs(this->_newSelRectRotationCenter.second);
      if (xCenter>0&&yCenter>0) {
	QPen pen(ROTATION_CENTER_LINES_COLOR,
		 2*QFeynCADEditor::_squareBaseSize+1);
	p.setPen(pen);
	p.drawLine(xCenter-3*QFeynCADEditor::_gridStep/2,yCenter,
		   xCenter+3*QFeynCADEditor::_gridStep/2,yCenter);
	p.drawLine(xCenter,yCenter-3*QFeynCADEditor::_gridStep/2,
		   xCenter,yCenter+3*QFeynCADEditor::_gridStep/2);
	p.setPen(ROTATION_CENTER_POINT_COLOR);
	p.setBrush(ROTATION_CENTER_POINT_COLOR);
	p.drawEllipse(xCenter-QFeynCADEditor::_circleBaseSize,
		      yCenter-QFeynCADEditor::_circleBaseSize,
		      2*QFeynCADEditor::_circleBaseSize+1,
		      2*QFeynCADEditor::_circleBaseSize+1);
      }
      if (this->_newSelRectRotationAngle) {
	// Si ruota la cornice di selezione rispetto al centro
	p.translate(xCenter,yCenter);
	xBase-=xCenter,yBase-=yCenter;
	p.rotate(this->_newSelRectRotationAngle);
      }
    }
    if (xBase>=0||yBase>=0||xBase+width>=0||yBase+height>=0) {
      QPen pen(black,2*QFeynCADEditor::_squareBaseSize);
      pen.setStyle(Qt::DotLine);
      p.setPen(pen);
      p.drawLine(xBase,yBase,xBase+width,yBase);
      p.drawLine(xBase,yBase,xBase,yBase+height);
      p.drawLine(xBase+width,yBase,xBase+width,yBase+height);
      p.drawLine(xBase,yBase+height,xBase+width,yBase+height);
    }
  }
  p.end();
  // And as our final act, we actually paint the pixmap onto the real widget
  p.begin(this);
  p.drawPixmap(start,pixmap);
  return;
}
#undef SMOOTH_PEN

#undef ENABLED_REFERENCE_COLOR
#undef DISABLED_REFERENCE_COLOR
#undef SELECTION_RECTANGLE_COLOR
#undef ENABLED_GRID_COLOR
#undef DISABLED_GRID_COLOR
#undef ENABLED_TAG_POINT_COLOR
#undef ENABLED_TAG_SQUARE_COLOR
#undef DISABLED_TAG_POINT_COLOR
#undef DISABLED_TAG_SQUARE_COLOR
#undef SELECTED_TAG_POINT_COLOR
#undef SELECTED_TAG_SQUARE_COLOR
#undef ENABLED_LINE_COLOR
#undef DISABLED_LINE_COLOR
#undef SELECTED_LINE_COLOR
#undef ENABLED_ARROW_COLOR
#undef DISABLED_ARROW_COLOR
#undef SELECTED_ARROW_COLOR
#undef ENABLED_VERTEX_COLOR
#undef DISABLED_VERTEX_COLOR
#undef SELECTED_VERTEX_COLOR
#undef ROTATION_CENTER_POINT_COLOR
#undef ROTATION_CENTER_LINES_COLOR

void QFeynCADEditor::setModeTo(QFeynCADEditor::mode m) {
  if (this->_mode==PasteMode)
    this->_accelerator->setEnabled(false);
  // Si azzerano eventuali selezioni
  this->_memoryOne=make_pair(-1,-1);
  this->_memoryTwo=make_pair(-1,-1);
  this->_selRectOne=make_pair(-1,-1);
  this->_selRectTwo=make_pair(-1,-1);
  this->_newSelRectOne=make_pair(-1,-1);
  this->_newSelRectTwo=make_pair(-1,-1);
  // Si fa comunque il merge dell'alpha channel
  this->mergeAlphaUniquifying();
  // Questo e' una merda, e vale solo per chiamate dall'esterno
  if (m==PasteMode) {
    this->_accelerator->setEnabled(true);
#define CLIPBOARD_TO_ALPHA(CENTER_X,CENTER_Y) \
    /* Case of a paste\
         Si azzera la selezione*/\
    this->_selRectOne=make_pair(-1,-1);\
    this->_selRectTwo=make_pair(-1,-1);\
    this->mergeAlphaUniquifying();\
    /* Si copia la clipboard nell'(_alpha)\
       Prima di tutto e' necessario traslare...*/\
    int traslX=CENTER_X-QFeynCADEditor::_clipboard.gridSize.first/2,\
      traslY=CENTER_Y-QFeynCADEditor::_clipboard.gridSize.second/2;\
    for (set<vertex>::const_iterator\
	     i=QFeynCADEditor::_clipboard.vertices.begin();\
	   i!=QFeynCADEditor::_clipboard.vertices.end();++i) {\
      vertex newVertex(*i);\
      newVertex.pos.first+=traslX;\
      newVertex.pos.second+=traslY;\
      this->_alpha.vertices.insert(newVertex);\
    }\
    for (multiset<line>::const_iterator\
	     i=QFeynCADEditor::_clipboard.lines.begin();\
	   i!=QFeynCADEditor::_clipboard.lines.end();++i) {\
      line newLine(*i);\
      newLine.vOne.first+=traslX;\
      newLine.vOne.second+=traslY;\
      newLine.vTwo.first+=traslX;\
      newLine.vTwo.second+=traslY;\
      newLine.selOne=true;\
      newLine.selTwo=true;\
      this->_alpha.lines.insert(newLine);\
    }\
    for (set<tag>::const_iterator\
	     i=QFeynCADEditor::_clipboard.tags.begin();\
	   i!=QFeynCADEditor::_clipboard.tags.end();++i) {\
      tag newTag(*i);\
      newTag.pos.first+=traslX;\
      newTag.pos.second+=traslY;\
      this->_alpha.tags.insert(newTag);\
    }\
    /*  Si crea il rettangolo di selezione*/\
    this->_newSelRectOne=\
      make_pair(CENTER_X-QFeynCADEditor::_clipboard.gridSize.first/2,\
		CENTER_Y-QFeynCADEditor::_clipboard.gridSize.second/2);\
    this->_newSelRectTwo=\
      make_pair(this->_newSelRectOne.first+\
		QFeynCADEditor::_clipboard.gridSize.first,\
		this->_newSelRectOne.second+\
	        QFeynCADEditor::_clipboard.gridSize.second);
  //
    // Deve essere la stessa condizione di (QFeynCADMain.C)
    if ((QFeynCADEditor::_clipboard.vertices.size()+
	 QFeynCADEditor::_clipboard.tags.size())) {
      CLIPBOARD_TO_ALPHA(this->_status.gridSize.first/2,
			 this->_status.gridSize.second/2)
    // We put the selection at the center of the widget
    this->_memoryOne=make_pair(this->_status.gridSize.first/2,
			       this->_status.gridSize.second/2);
    this->_memoryTwo=
      make_pair(QFeynCADEditor::_clipboard.gridSize.first,
		QFeynCADEditor::_clipboard.gridSize.second);
    }
  }
  // Deve andare prima, sia per il colore della griglia che per il segnale
  this->_mode=m;
  emit newMode();
  this->repaint();
  return;
}

void QFeynCADEditor::mousePressEvent(QMouseEvent* e) {
  pair<int,int> gridPos(xGridPosWhenXCanvasIs(e->x()),
			yGridPosWhenYCanvasIs(e->y()));
  switch (this->_mode) {
  case DrawMode:
    switch (e->button()) {
    case LeftButton: {
      set<vertex>::iterator i=this->_status.vertices.find(vertex(gridPos));
      // Se il vertice non c'e' gia' lo si inserisce, altrimenti
      //  lo si copia nell'(_alpha) insieme a tutte le linee a lui collegate
      //  ed eventualmente lo si modifica
      if (i!=this->_status.vertices.end()) {
	for (multiset<line>::iterator
	       j=this->_status.lines.begin(),k;j!=this->_status.lines.end();)
	  if (j->vOne==i->pos||j->vTwo==i->pos) {
	    this->_alpha.lines.insert(*j);
	    this->_status.lines.erase(k=j++);
	  } else
	    ++j;
	//
	this->_alpha.vertices.insert(*i);
	this->_status.vertices.erase(i);
	//
	this->repaint();
	this->_memoryOne=gridPos;
	emit setMainAppEnabledReq(false);
	this->_modifyOrDelete->setEnabled(true);
	this->_modifyOrDelete->exec(QCursor::pos());
      } else {
	vertex newVertex(QFeynCADEditor::_vertex);
	newVertex.pos=gridPos;
	this->_status.vertices.insert(newVertex);
	this->repaint();
	this->_last=Vertex,this->_lastOne=gridPos;
	ostringstream descr;
	descr << "Add Vertex at (" << gridPos.first << "," << gridPos.second
	      << ")";
	this->pushHistory(descr.str());
	this->updatePostscript();
      }
      break;
    }
    case MidButton:
      if (this->_status.vertices.find(vertex(gridPos))!=
	  this->_status.vertices.end())
	{ this->_tracingLine=true; this->_memoryOne=gridPos; }
      else
	this->_tracingLine=false;
      break;
    case RightButton: {
      set<tag>::iterator i=this->_status.tags.find(tag(gridPos));
      // Se la tag non c'e' gia' la si inserisce,
      //  altrimenti la si evidenzia ed eventualmente la si modifica
      if (i!=this->_status.tags.end()) {
	this->_alpha.tags.insert(*i);
	this->_status.tags.erase(i);
	this->repaint();
	this->_memoryOne=gridPos;
	emit setMainAppEnabledReq(false);
	this->_modifyOrDelete->setEnabled(true);
	this->_modifyOrDelete->exec(QCursor::pos());
      } else {
	tag newTag(QFeynCADEditor::_tag);
	newTag.pos=gridPos;
	this->_status.tags.insert(newTag);
	this->repaint();
	this->_last=Tag,this->_lastOne=gridPos;
	ostringstream descr;
	descr << "Add Tag at (" << gridPos.first << "," << gridPos.second
	      << ")";
	this->pushHistory(descr.str());
	this->updatePostscript();
      }
      break;
    }
    default:
      break;
    }
    break;
  case SelectMode:
    switch (e->button()) {
    case LeftButton:
      if (this->_selRectOne!=make_pair(-1,-1)) {
	// A previous selection is present
	if (gridPos.first>=min(this->_selRectOne.first,
			       this->_selRectTwo.first)&&
	    gridPos.first<=max(this->_selRectOne.first,
			       this->_selRectTwo.first)&&
	    gridPos.second>=min(this->_selRectOne.second,
				this->_selRectTwo.second)&&
	    gridPos.second<=max(this->_selRectOne.second,
				this->_selRectTwo.second)) {
	  // The mouse is in the middle of the selection: start moving
	  this->_memoryOne=gridPos;
	  this->_memoryTwo=gridPos;
	  return;
	} else {
	  // We have to zero the old selection
	  this->_selRectOne=make_pair(-1,-1);
	  this->_selRectTwo=make_pair(-1,-1);
	  // this->_newSelRectXxx are already (-1,-1) here
	  // We unselect, checking that there are no collisions
	  if (this->_alpha.size()) {
	    // The selection is not empty
	    bool collisions=this->mergeAlphaUniquifying();
	    ostringstream descr;
	    descr << "Merge selection";
	    this->pushHistory(descr.str());
	    if (collisions) {
	      // Lo scopo e' eliminare il baco della falsa selezione
	      //  Per buona misura si disattiva tutto il possibile
	      this->_memoryOne=make_pair(-1,-1);
	      this->_memoryTwo=make_pair(-1,-1);
	      this->_newSelRectOne=make_pair(-1,-1);
	      this->_newSelRectTwo=make_pair(-1,-1);
	      this->repaint();
	      break;
	    }
	  }
	}
      }
      // We start a new selection
      this->_memoryOne=make_pair(e->x(),e->y());
      this->_memoryTwo=make_pair(-1,-1);
      this->_selRectOne=gridPos;
      this->_selRectTwo=make_pair(-1,-1);
      this->_newSelRectOne=gridPos;
      this->_newSelRectTwo=make_pair(-1,-1);
      this->repaint();
      break;
    case MidButton:
      {
	// Questa condizione deve essere la stessa di (QFeynCADMain.C)
	if ((QFeynCADEditor::_clipboard.vertices.size()+
	     QFeynCADEditor::_clipboard.tags.size())) {
          CLIPBOARD_TO_ALPHA(gridPos.first,gridPos.second)
	  //  Si entra (temporaneamente) nel (PasteMode)
	  //   (_memoryOne) contiene il punto di partenza,
	  //   (_memoryTwo) la dimensione del box
	  this->repaint();
	  this->_memoryOne=gridPos;
	  this->_memoryTwo=
	    make_pair(QFeynCADEditor::_clipboard.gridSize.first,
		      QFeynCADEditor::_clipboard.gridSize.second);
	  //  Questo e' l'unico caso in cui non si puo' chiamare (setModeTo)
	  this->_mode=PasteMode;
	  emit newMode();
	  this->_accelerator->setEnabled(true);
	}
      }
      break;
    case RightButton:
      break;
    default:
      break;
    }
    break;
  case PasteMode:
    switch (e->button())
      { default: this->acceptPaste(); break; }
    break;
  default:
    // Unsupported modes
    break;
  }
  return;
}
void QFeynCADEditor::mouseMoveEvent(QMouseEvent* e) {
  pair<int,int> gridPos(xGridPosWhenXCanvasIs(e->x()),
			yGridPosWhenYCanvasIs(e->y()));
  switch (this->_mode) {
  case DrawMode:
    if (this->_tracingLine) {
      this->_memoryTwo=gridPos;
      QPaintEvent event(this->getEnvelope(this->_memoryOne,this->_memoryTwo));
      this->paintEvent(&event);
    }
    break;
  case SelectMode:
    if (this->_selRectOne!=make_pair(-1,-1))
      if (this->_selRectTwo==make_pair(-1,-1)) {
	// Selecting
	//  Si plotta la cornice che diventera' il rettangolo di selezione
	if (abs(e->x()-this->_memoryOne.first)>
	    (3*QFeynCADEditor::_gridStep/8)||
	    abs(e->y()-this->_memoryOne.second)>
	    (3*QFeynCADEditor::_gridStep/8))
	  this->_newSelRectTwo=gridPos;
	QPaintEvent
	  event(this->getEnvelope(this->_selRectOne,this->_selRectTwo));
	this->paintEvent(&event);
      } else if (this->_memoryOne!=make_pair(-1,-1)) {
	// Move
	//  Si calcola il vettore di traslazione per gli oggetti
	int xObjTransl=gridPos.first-this->_memoryTwo.first,
	  yObjTransl=gridPos.second-this->_memoryTwo.second;
	//  Si trasla la selezione
	this->translateAlpha(xObjTransl,yObjTransl);
	//  Si memorizza l'attuale posizione del mouse
	this->_memoryTwo=gridPos;
	//  Si calcola il vettore di traslazione per il rettangolo di selezione
	int xSelTransl=gridPos.first-this->_memoryOne.first,
	  ySelTransl=gridPos.second-this->_memoryOne.second;
	this->_newSelRectOne=make_pair(this->_selRectOne.first+xSelTransl,
				       this->_selRectOne.second+ySelTransl);
	this->_newSelRectTwo=make_pair(this->_selRectTwo.first+xSelTransl,
				       this->_selRectTwo.second+ySelTransl);
	QPaintEvent
	  event(this->getEnvelope(this->_newSelRectOne,this->_newSelRectTwo));
	this->paintEvent(&event);
      }
    break;
  case PasteMode:
    {
      // Qui (_memoryTwo) contiene le dimensioni del rettangolo di selezione
      this->_newSelRectOne=make_pair(gridPos.first-this->_memoryTwo.first/2,
				     gridPos.second-this->_memoryTwo.second/2);
      this->_newSelRectTwo=make_pair(this->_newSelRectOne.first+
				     this->_memoryTwo.first,
				     this->_newSelRectOne.second+
				     this->_memoryTwo.second);
      // Si calcola il vettore di traslazione per l'(_alpha)
      int xTransl=gridPos.first-this->_memoryOne.first,
	yTransl=gridPos.second-this->_memoryOne.second;
      // Necessary to avoid runaway selections :)
      this->_memoryOne=gridPos;
      this->translateAlpha(xTransl,yTransl);
      QPaintEvent
	event(this->getEnvelope(this->_newSelRectOne,this->_newSelRectTwo));
      this->paintEvent(&event);
    }
    break;
  default:
    // Unsupported modes
    break;
  }
  emit newGridPosition(gridPos.first,gridPos.second);
  return;
}
#define PROTO_LINE_OR_TADPOLE(a,b) ((a==b)?line(a):line(a,b))
void QFeynCADEditor::mouseReleaseEvent(QMouseEvent* e) {
  pair<int,int> gridPos(xGridPosWhenXCanvasIs(e->x()),
			yGridPosWhenYCanvasIs(e->y()));
  switch (this->_mode) {
  case DrawMode:
    switch (e->button()) {
    case MidButton:
      if (this->_tracingLine) {
	this->_tracingLine=false;
	if (this->_status.vertices.find(vertex(gridPos))!=
	    this->_status.vertices.end())
	  // Se la linea non c'e' gia' la si inserisce, altrimenti
	  //  la si copia nell'(_alpha) ed eventualmente la si modifica
	  if (this->_status.lines.
	      find(PROTO_LINE_OR_TADPOLE(this->_memoryOne,gridPos))!=
	      this->_status.lines.end()||
	      this->_status.lines.
	      find(PROTO_LINE_OR_TADPOLE(gridPos,this->_memoryOne))!=
	      this->_status.lines.end()) {
	    this->_memoryTwo=gridPos;
	    pair<multiset<line>::iterator,multiset<line>::iterator>
	      range;
	    range=this->_status.lines.
	      equal_range(PROTO_LINE_OR_TADPOLE(this->_memoryOne,
						this->_memoryTwo));
	    for (multiset<line>::iterator i=range.first,j;i!=range.second;) {
	      this->_alpha.lines.insert(*i);
	      this->_status.lines.erase(j=i++);
	    }
	    range=this->_status.lines.
	      equal_range(PROTO_LINE_OR_TADPOLE(this->_memoryTwo,
						this->_memoryOne));
	    for (multiset<line>::iterator i=range.first,j;i!=range.second;) {
	      this->_alpha.lines.insert(*i);
	      this->_status.lines.erase(j=i++);
	    }
	    QPaintEvent
	      event(this->getEnvelope(this->_memoryOne,this->_memoryTwo));
	    this->paintEvent(&event);
	    emit setMainAppEnabledReq(false);
	    this->_modifyOrDelete->setEnabled(true);
	    this->_modifyOrDelete->exec(QCursor::pos());
	  } else {
	    line newLine(this->_memoryOne!=gridPos?
			 QFeynCADEditor::_line:QFeynCADEditor::_tadpole);
	    newLine.vOne=this->_memoryOne,newLine.vTwo=gridPos;
	    this->_status.lines.insert(newLine);
	    this->repaint();
	    this->_last=Line,
	      this->_lastOne=this->_memoryOne,this->_lastTwo=gridPos;
	    ostringstream descr;
	    descr << "Add ";
	    if (gridPos!=this->_memoryOne)
	      descr << "Line between (" << this->_memoryOne.first << ","
		    << this->_memoryOne.second << ") and (" << gridPos.first
		    << "," << gridPos.second << ")";
	    else
	      descr << "Tadpole at (" << this->_memoryOne.first << ","
		    << this->_memoryOne.second << ")";
	    this->pushHistory(descr.str());
	    this->updatePostscript();
	  }
	else
	  this->repaint();
      }
      break;
    default:
      break;
    }
    break;
  case SelectMode:
    switch (e->button()) {
    case LeftButton:
      // Si disattiva la cornice di selezione
      this->_newSelRectOne=make_pair(-1,-1);
      this->_newSelRectTwo=make_pair(-1,-1);
      if (this->_selRectTwo==make_pair(-1,-1)) {
	// Case of a new selection
	int xDist=abs(e->x()-this->_memoryOne.first),
	  yDist=abs(e->y()-this->_memoryOne.second);
	//  Cleanup dei marker
	this->_memoryOne=make_pair(-1,-1);
	this->_memoryTwo=make_pair(-1,-1);
	//
	if (this->_selRectOne==make_pair(-1,-1)||
	    (xDist<=(3*QFeynCADEditor::_gridStep/8)&&
	     yDist<=(3*QFeynCADEditor::_gridStep/8))) {
	  // Selezione vuota: si disattiva tutto
	  this->_selRectOne=make_pair(-1,-1);
	  this->_selRectTwo=make_pair(-1,-1);
	  this->repaint();
	  emit newSelection();
	} else {
	  this->_selRectTwo=gridPos;
	  //  Si spostano gli oggetti compresi dal rettangolo di selezione
	  //   nell'(_alpha)
	  int xl=min(this->_selRectOne.first,this->_selRectTwo.first),
	    xr=max(this->_selRectOne.first,this->_selRectTwo.first),
	    yb=min(this->_selRectOne.second,this->_selRectTwo.second),
	    yt=max(this->_selRectOne.second,this->_selRectTwo.second);
	  for (set<vertex>::iterator i=this->_status.vertices.begin(),j;
	       i!=this->_status.vertices.end();)
	    if (i->pos.first>=xl&&i->pos.first<=xr&&
		i->pos.second>=yb&&i->pos.second<=yt) {
	      this->_alpha.vertices.insert(*i);
	      this->_status.vertices.erase(j=i++);
	    } else
	      ++i;
	  for (multiset<line>::iterator i=this->_status.lines.begin(),j;
	       i!=this->_status.lines.end();) {
	    bool first=(i->vOne.first>=xl&&i->vOne.first<=xr&&
			i->vOne.second>=yb&&i->vOne.second<=yt),
	      second=(i->vTwo.first>=xl&&i->vTwo.first<=xr&&
		      i->vTwo.second>=yb&&i->vTwo.second<=yt);
	    if (first||second) {
	      line newLine(*i);
	      if (first)
		newLine.selOne=true;
	      if (second)
		newLine.selTwo=true;
	      this->_alpha.lines.insert(newLine);
	      this->_status.lines.erase(j=i++);
	    } else
	      ++i;
	  }
	  for (set<tag>::iterator i=this->_status.tags.begin(),j;
	       i!=this->_status.tags.end();)
	    if (i->pos.first>=xl&&i->pos.first<=xr&&
		i->pos.second>=yb&&i->pos.second<=yt) {
	      this->_alpha.tags.insert(*i);
	      this->_status.tags.erase(j=i++);
	    } else
	      ++i;
	  this->repaint();
	  this->updateVertexList();
	  this->updateLineList();
	  this->updateTagList();
	  //  Si crea la selezione
	  status selection;
	  selection.gridSize.first=xr-xl;
	  selection.gridSize.second=yt-yb;
	  //   Si copiano gli oggetti selezionati, traslandoli
	  //    Le linee "orfane" non vengono copiate
	  for (set<vertex>::const_iterator i=this->_alpha.vertices.begin();
	       i!=this->_alpha.vertices.end();++i) {
	    vertex newVertex(*i);
	    newVertex.pos.first-=xl;
	    newVertex.pos.second-=yb;
	    selection.vertices.insert(newVertex);
	  }
	  for (multiset<line>::const_iterator i=this->_alpha.lines.begin();
	       i!=this->_alpha.lines.end();++i)
	    // We want only lines which are entirely contained in (_alpha)
	    if (i->selOne&&i->selTwo) {
	      line newLine(*i);
	      newLine.vOne.first-=xl;
	      newLine.vOne.second-=yb;
	      newLine.vTwo.first-=xl;
	      newLine.vTwo.second-=yb;
	      selection.lines.insert(newLine);
	    }
	  for (set<tag>::const_iterator i=this->_alpha.tags.begin();
	       i!=this->_alpha.tags.end();++i) {
	    tag newTag(*i);
	    newTag.pos.first-=xl;
	    newTag.pos.second-=yb;
	    selection.tags.insert(newTag);
	  }
	  //  Se la selezione e' non vuota la si trasmette alla clipboard,
	  //   attivando comunque i menu per (paste) and (transform)
	  if (selection.vertices.size()+selection.tags.size())
	    QFeynCADEditor::_clipboard=selection;
	  emit newSelection();
	  //  Si trasmette la variazione a (_modifySelection)
	  if (this->_modifySelection) {
	    this->_modifySelection->newBoundariesAre
	      (min(this->_selRectOne.first,this->_selRectTwo.first),
	       max(this->_selRectOne.first,this->_selRectTwo.first),
	       min(this->_selRectOne.second,this->_selRectTwo.second),
	       max(this->_selRectOne.second,this->_selRectTwo.second));
	    // Si calcola il bounding box effettivo
	    if (this->_status.vertices.size()+this->_alpha.vertices.size()+
		this->_status.tags.size()+this->_alpha.tags.size()) {
	      int xmin=numeric_limits<int>::max(),
		xmax=numeric_limits<int>::min();
	      int ymin=xmin,ymax=xmax;
	      for (set<vertex>::const_iterator
		     i=this->_status.vertices.begin();
		   i!=this->_status.vertices.end();++i)
		xmin=min(xmin,i->pos.first),xmax=max(xmax,i->pos.first),
		  ymin=min(ymin,i->pos.second),ymax=max(ymax,i->pos.second);
	      for (set<vertex>::const_iterator i=this->_alpha.vertices.begin();
		   i!=this->_alpha.vertices.end();++i)
		xmin=min(xmin,i->pos.first),xmax=max(xmax,i->pos.first),
		  ymin=min(ymin,i->pos.second),ymax=max(ymax,i->pos.second);
	      for (set<tag>::const_iterator i=this->_status.tags.begin();
		   i!=this->_status.tags.end();++i)
		xmin=min(xmin,i->pos.first),xmax=max(xmax,i->pos.first),
		  ymin=min(ymin,i->pos.second),ymax=max(ymax,i->pos.second);
	      for (set<tag>::const_iterator i=this->_alpha.tags.begin();
		   i!=this->_alpha.tags.end();++i)
		xmin=min(xmin,i->pos.first),xmax=max(xmax,i->pos.first),
		  ymin=min(ymin,i->pos.second),ymax=max(ymax,i->pos.second);
	      this->_modifySelection->realBoundingBoxIs(xmin,xmax,ymin,ymax);
	    } else
	      this->_modifySelection->realBoundingBoxIs(0,0,0,0);
	  }
	}
      } else {
	// Move
	//  Si sposta il rettangolo di selezione
	int xTransl=this->_memoryTwo.first-this->_memoryOne.first;
	int yTransl=this->_memoryTwo.second-this->_memoryOne.second;
	//  Cleanup dei marker
	this->_memoryOne=make_pair(-1,-1);
	this->_memoryTwo=make_pair(-1,-1);
	// Si controlla che il rettangolo non sia stato riposizionato dove era
	if (xTransl||yTransl) {
	  this->_selRectOne.first+=xTransl,this->_selRectOne.second+=yTransl;
	  this->_selRectTwo.first+=xTransl,this->_selRectTwo.second+=yTransl;
	  if (this->_alpha.size()) {
	    //  Aggiornamento dei combo-box
	    this->updateVertexList();
	    this->updateLineList();
	    this->updateTagList();
	    //
	    this->updatePostscript();
	  }
	}
	this->repaint();
      }
      break;
    case MidButton:
      break;
    case RightButton:
      break;
    default:
      break;
    }
    break;
  default:
    // Unsupported Modes
    break;
  }
  return;
}

void QFeynCADEditor::updateModifyOrDelete() {
  // Cleanup
  this->_modifyOrDelete->clear();
  this->_modifyOrDeleteTitle=new QLabel(this->_modifyOrDelete);
  QFont modifyOrDeleteTitleFont(this->_modifyOrDeleteTitle->font());
  modifyOrDeleteTitleFont.setBold(true);
  this->_modifyOrDeleteTitle->setFont(modifyOrDeleteTitleFont);
  // Si cerca cosa e' stato evidenziato
  if (this->_alpha.vertices.size()) {
    ostringstream menuHead;
    menuHead << "Vertex at (" << this->_alpha.vertices.begin()->pos.first
	     << "," << this->_alpha.vertices.begin()->pos.second << ")";
    this->_modifyOrDeleteTitle->setText(menuHead.str().c_str());
    this->_modifyOrDelete->insertItem(this->_modifyOrDeleteTitle);
    this->_modifyOrDelete->insertSeparator();
    this->_modifyOrDelete->
      insertItem("Modify...",this,SLOT(prepareModifyVertex()));
    this->_modifyOrDelete->insertItem("Delete",this,SLOT(deleteVertex()));
    this->_modifyVertex->setTo(*this->_alpha.vertices.begin());
    this->updateVertexList();
    // C'e' al massimo un solo vertice evidenziato
  } else if (this->_alpha.lines.size()) {
#define REBUILD_MODIFY_LINE_PARENT \
    /* Cleanup */\
    /*  Questo distrugge automaticamente anche i figli */\
    delete this->_modifyLineParent;\
    this->_modifyLine.clear();\
    /**/\
    this->_modifyLineParent=new QFeynCADTabDialog(0);\
    this->_modifyLineParent->\
      setSizePolicy(QSizePolicy(QSizePolicy::Minimum,QSizePolicy::Minimum,0,0,\
				this->sizePolicy().hasHeightForWidth()));\
    this->_modifyLineParent->layout()->setResizeMode(QLayout::Fixed);\
    connect(this->_modifyLineParent,SIGNAL(applyButtonPressed()),\
	    this,SLOT(modifyLines()));\
    connect(this->_modifyLineParent,SIGNAL(cancelButtonPressed()),\
	    this,SLOT(mergeAlphaUniquifying()));\
    connect(this->_modifyLineParent,SIGNAL(cancelButtonPressed()),\
	    this,SLOT(updateLineList()));\
    connect(this->_modifyLineParent,SIGNAL(cancelButtonPressed()),\
	    this,SLOT(stopRunawayDialogCatcher()));
    //
    REBUILD_MODIFY_LINE_PARENT
    register unsigned count=0;
    for (multiset<line>::const_iterator i=this->_alpha.lines.begin();
	 i!=this->_alpha.lines.end();++i,++count) {
      if (!count) {
	ostringstream menuHead;
	if (i->vOne==i->vTwo)
	  menuHead << "Tadpole" << (this->_alpha.lines.size()==1?"":"s")
		   << " on (" << i->vOne.first << "," << i->vOne.second << ")";
	else
	  menuHead << "Line" << (this->_alpha.lines.size()==1?"":"s")
		   << " from (" << i->vOne.first << "," << i->vOne.second
		   << ") to (" << i->vTwo.first << "," << i->vTwo.second
		   << ")";
	this->_modifyOrDeleteTitle->setText(menuHead.str().c_str());
	this->_modifyOrDelete->insertItem(this->_modifyOrDeleteTitle);
	this->_modifyOrDelete->insertSeparator();
	this->_modifyOrDelete->insertItem("Add",this,SLOT(addLine()));
	this->_modifyOrDelete->
	  insertItem("Modify...",this,SLOT(prepareModifyLines()));
	this->_modifyOrDelete->
	  insertItem("Delete",this,SLOT(deleteLines()));
      }
#define REBUILD_MODIFY_LINE_ITEM(i,count) {\
        this->_modifyLine.\
	  push_back(pair<QFeynCADLineEdit*,QFeynCADTadpoleEdit*>(0,0));\
        if (i->vOne==i->vTwo) {\
	  this->_modifyLine[count].first=0;\
	  this->_modifyLine[count].second=\
	    new QFeynCADTadpoleEdit(this->_modifyLineParent);\
	  connect(this->_modifyLine[count].second,SIGNAL(activateApplyReq()),\
		  this->_modifyLineParent,SLOT(activateApply()));\
	  connect(this->_modifyLine[count].second,SIGNAL(normalized()),\
		  this->_modifyLineParent,SLOT(deactivateApply()));\
	  this->_modifyLine[count].second->setTo(*i);\
	  ostringstream title;\
	  title << "Selected Tadpole " << count+1;\
	  this->_modifyLineParent->addTab(this->_modifyLine[count].second,\
					  title.str().c_str());\
        } else {\
	  this->_modifyLine[count].first=\
	    new QFeynCADLineEdit(this->_modifyLineParent);\
	  this->_modifyLine[count].second=0;\
	  connect(this->_modifyLine[count].first,SIGNAL(activateApplyReq()),\
		  this->_modifyLineParent,SLOT(activateApply()));\
	  connect(this->_modifyLine[count].first,SIGNAL(normalized()),\
		  this->_modifyLineParent,SLOT(deactivateApply()));\
	  this->_modifyLine[count].first->setTo(*i);\
	  ostringstream title;\
	  title << "Selected Line " << count+1;\
	  this->_modifyLineParent->addTab(this->_modifyLine[count].first,\
					  title.str().c_str());\
        }\
      }
      //
      REBUILD_MODIFY_LINE_ITEM(i,count)
    }
    this->updateLineList();
  } else {
    // A questo punto la cosa da modificare deve per forza essere una tag
    if (!this->_alpha.tags.size())
      abort(); // Agonizing pain
    ostringstream menuHead;
    menuHead << "Tag at (" << this->_alpha.tags.begin()->pos.first << ","
	     << this->_alpha.tags.begin()->pos.second << ")";
    this->_modifyOrDeleteTitle->setText(menuHead.str().c_str());
    this->_modifyOrDelete->insertItem(this->_modifyOrDeleteTitle);
    this->_modifyOrDelete->insertSeparator();
    this->_modifyOrDelete->
      insertItem("Modify...",this,SLOT(prepareModifyTag()));
    this->_modifyOrDelete->insertItem("Delete",this,SLOT(deleteTag()));
    this->_modifyTag->setTo(*this->_alpha.tags.begin());
    this->updateTagList();
    // C'e' al massimo una sola tag evidenziata
  }
  return;
}
void QFeynCADEditor::updateVertexList() {
  QStringList vertexList;
  unsigned maxLen=0,len;
  set<vertex,vertex::depthLess> depthVertices;
  for (set<vertex>::const_iterator i=this->_status.vertices.begin();
       i!=this->_status.vertices.end();++i) {
    depthVertices.insert(*i);
    len=measure(i->pos.first)+measure(i->pos.second);
    maxLen=max(len,maxLen);
  }
  for (set<vertex>::const_iterator i=this->_alpha.vertices.begin();
       i!=this->_alpha.vertices.end();++i) {
    depthVertices.insert(*i);
    len=measure(i->pos.first)+measure(i->pos.second);
    maxLen=max(len,maxLen);
  }
  for (set<vertex,vertex::depthLess>::const_iterator
	 i=depthVertices.begin(),j;i!=depthVertices.end();++i) {
    ostringstream vertexDescr;
    bool isSelected=
      (this->_alpha.vertices.find(*i)!=this->_alpha.vertices.end());
    string sSize=QString::number(i->size).ascii();
    if (i==depthVertices.begin()||i->depth!=(--(j=i))->depth)
      vertexDescr << "@" << setw(2) << i->depth << " ";
    else
      vertexDescr << "    ";
    vertexDescr << (isSelected?"*":"(") << i->pos.first << ","
		<< i->pos.second << (isSelected?"*":")")
		<< string(maxLen-measure(i->pos.first)-
			  measure(i->pos.second)+1,' ') << setw(9) << left
		<< (i->type.size()>9?
		    (i->type.substr(0,8)+"+").c_str():i->type.c_str())
		<< " " << setw(4) << left
		<< (sSize.size()>4?
		    (sSize.substr(0,3)+"+").c_str():sSize.c_str());
    vertexList+=QString(vertexDescr.str().c_str());
  }
  emit vertexListUpdateReq(vertexList);  
  return;
}
void QFeynCADEditor::updateLineList() {
  QStringList lineGroupList;
  unsigned maxLenOne=0,maxLenTwo=0;
  multiset<line,line::depthLess> depthLines;
  for (multiset<line>::const_iterator i=this->_status.lines.begin();
       i!=this->_status.lines.end();++i) {
    depthLines.insert(*i);
    maxLenOne=max(measure(i->vOne.first)+measure(i->vOne.second),maxLenOne);
    maxLenTwo=max(measure(i->vTwo.first)+measure(i->vTwo.second),maxLenTwo);
  }
  for (multiset<line>::const_iterator i=this->_alpha.lines.begin();
       i!=this->_alpha.lines.end();++i) {
    line newLine(*i);
    // Bieco trucco per marcare le linee selezionate
    newLine.selOne=true;
    depthLines.insert(newLine);
    maxLenOne=max(measure(i->vOne.first)+measure(i->vOne.second),maxLenOne);
    maxLenTwo=max(measure(i->vTwo.first)+measure(i->vTwo.second),maxLenTwo);
  }
  for (multiset<line,line::depthLess>::const_iterator
	 i=depthLines.begin(),j;i!=depthLines.end();++i) {
    ostringstream lineGroupDescr;
    bool depthChange=(i==depthLines.begin()||i->depth!=(--(j=i))->depth),
      isSelected=i->selOne;
    // We skip over all other members of the class
    unsigned counter=0;
    j=i;
    do {
      if (j->depth==i->depth&&j->vOne==i->vOne&&j->vTwo==i->vTwo)
	++counter;
      else
	break;
    } while ((++j)!=depthLines.end());
    // Questo va bene anche nel caso della terminazione
    i=(--j);
    //
    if (depthChange)
      lineGroupDescr << "@" << setw(2) << i->depth << " ";
    else
      lineGroupDescr << "    ";
    if (i->vTwo!=i->vOne)
      // Alla fine tutto questo avra' lunghezza (maxLenOne+3)...
      lineGroupDescr << string(maxLenOne-measure(i->vOne.first)-
			       measure(i->vOne.second),' ')
		     << (isSelected?"*":"(") << i->vOne.first << ","
		     << i->vOne.second << (isSelected?"*":")");
    else
      //  ...ed idem questo
      lineGroupDescr << string(maxLenOne-2,' ') << "Tad. ";
    lineGroupDescr << (isSelected?"*":"(") << i->vTwo.first << ","
		   << i->vTwo.second << (isSelected?"*":")")
		   << string(maxLenTwo-measure(i->vTwo.first)-
			     measure(i->vTwo.second)+1,' ');
    if (counter>1)
      lineGroupDescr << "[x" << counter << "]";
    else
      lineGroupDescr << setw(7) << left
		     << (i->type.size()>7?
			 (i->type.substr(0,6)+"+").c_str():i->type.c_str());
    lineGroupList+=QString(lineGroupDescr.str().c_str());
  }
  emit lineListUpdateReq(lineGroupList);  
  return;
}
void QFeynCADEditor::updateTagList() {
  QStringList tagList;
  unsigned maxLen=0,len;
  set<tag,tag::depthLess> depthTags;
  for (set<tag>::const_iterator i=this->_status.tags.begin();
       i!=this->_status.tags.end();++i) {
    depthTags.insert(*i);
    len=measure(i->pos.first)+measure(i->pos.second);
    maxLen=max(len,maxLen);
  }
  for (set<tag>::const_iterator i=this->_alpha.tags.begin();
       i!=this->_alpha.tags.end();++i) {
    depthTags.insert(*i);
    len=measure(i->pos.first)+measure(i->pos.second);
    maxLen=max(len,maxLen);
  }
  for (set<tag,tag::depthLess>::const_iterator
	 i=depthTags.begin(),j;i!=depthTags.end();++i) {
    ostringstream tagDescr;
    bool isSelected=(this->_alpha.tags.find(*i)!=this->_alpha.tags.end());
    if (i==depthTags.begin()||i->depth!=(--(j=i))->depth)
      tagDescr << "@" << setw(2) << i->depth << " ";
    else
      tagDescr << "    ";
    tagDescr << (isSelected?"*":"(") << i->pos.first << "," << i->pos.second
	     << (isSelected?"*":")")
	     << string(maxLen-measure(i->pos.first)-
		       measure(i->pos.second)+1,' ') << setw(14) << left
	     << (i->text.size()>12?
		 (string("\"")+i->text.substr(0,11)+"\"+").c_str():
		 (string("\"")+i->text+"\"").c_str());
    tagList+=QString(tagDescr.str().c_str());
  }
  emit tagListUpdateReq(tagList);  
  return;
}

void QFeynCADEditor::selectVertexWhenListIs(int index) {
  if (this->_mode==DrawMode) {
    // We don't want any selection around
    this->mergeAlphaUniquifying();
    //
    register int count=0;
    set<vertex,vertex::depthLess> depthVertices;
    for (set<vertex>::const_iterator i=this->_status.vertices.begin();
	 i!=this->_status.vertices.end();++i)
      depthVertices.insert(*i);
    for (set<vertex,vertex::depthLess>::const_iterator
	   i=depthVertices.begin();i!=depthVertices.end();++i,++count)
      if (count==index) {
	this->_memoryOne=i->pos;
	this->_modifyVertex->setTo(*i);
	this->prepareModifyVertex();
	return;
      }
    // Peccato, niente classe
    abort(); // Agonizing pain
    return;
  }
  return;
}
void QFeynCADEditor::selectLineWhenListIs(int index) {
  if (this->_mode==DrawMode) {
    // We don't want any selection around
    this->mergeAlphaUniquifying();
    //
    register int count=0;
    multiset<line,line::depthLess> depthLines;
    for (multiset<line>::const_iterator i=this->_status.lines.begin();
	 i!=this->_status.lines.end();++i)
      depthLines.insert(*i);
    for (multiset<line,line::depthLess>::const_iterator
	   i=depthLines.begin(),j;i!=depthLines.end();++i,++count) {
      if (count==index) {
	// La classe e' quella giusta
	//  Si cercano gli elementi in questione nella vera lista
	pair<multiset<line>::iterator,multiset<line>::iterator>
	  range=this->_status.lines.equal_range(*i);
	//  A questo punto non siamo pero' sicuri
	//   che (depth) sia quella voluta
	for (multiset<line>::iterator k=range.first,l;k!=range.second;)
	  if (k->depth==i->depth) {
	    this->_alpha.lines.insert(*k);
	    this->_status.lines.erase(l=k++);
	  } else
	    ++k;
	//  Aggiornamenti vari
	this->_memoryOne=i->vOne;
	this->_memoryTwo=i->vTwo;
	this->updateModifyOrDelete();
	//  Qui non si puo' chiamare (prepareModifyLines()), perche' altrimenti
	//   vengono evidenziate troppe linee
	emit setMainAppEnabledReq(false);
	this->_modifyLineParent->setEnabled(true);
	ostringstream descr;
	descr << "Modifying ";
	if (this->_memoryTwo!=this->_memoryOne)
	  descr << "Lines between (" << this->_memoryOne.first << ","
		<< this->_memoryOne.second << ") and ("
		<< this->_memoryTwo.first << "," << this->_memoryTwo.second
		<< ")";
	else
	  descr << "Tadpoles at (" << this->_memoryOne.first << ","
		<< this->_memoryOne.second << ")";
	descr << "...";
	emit message(true,descr.str());
	this->repaint();
	this->_modifyLineParent->deactivateApply();
	this->_modifyLineParent->show();
	this->_runawayDialogCatcher->start(100);
	return;
      }
      // We skip over all other members of the class
      j=i;
      do
	if (j->depth!=i->depth||j->vOne!=i->vOne||j->vTwo!=i->vTwo)
	  break;
      while ((++j)!=depthLines.end());
      // Questo va bene anche nel caso della terminazione
      i=(--j);
      //
    }
    // Peccato, niente classe
    abort(); // Agonizing pain
    return;
  }
  return;
}
void QFeynCADEditor::selectTagWhenListIs(int index) {
  if (this->_mode==DrawMode) {
    // We don't want any selection around
    this->mergeAlphaUniquifying();
    //
    register int count=0;
    set<tag,tag::depthLess> depthTags;
    for (set<tag>::const_iterator i=this->_status.tags.begin();
	 i!=this->_status.tags.end();++i)
      depthTags.insert(*i);
    for (set<tag,tag::depthLess>::const_iterator
	   i=depthTags.begin();i!=depthTags.end();++i,++count)
      if (count==index) {
	this->_memoryOne=i->pos;
	this->_modifyTag->setTo(*i);
	this->prepareModifyTag();
	return;
      }
    // Peccato, niente classe
    abort(); // Agonizing pain
    return;
  }
  return;
}

void QFeynCADEditor::addLine() {
  line newLine(this->_memoryOne!=this->_memoryTwo?
	       QFeynCADEditor::_line:QFeynCADEditor::_tadpole);
  newLine.vOne=this->_memoryOne,newLine.vTwo=this->_memoryTwo;
  this->_status.lines.insert(newLine);
  this->repaint();
  this->_last=Line,
    this->_lastOne=this->_memoryOne,this->_lastTwo=this->_memoryTwo;
  ostringstream descr;
  descr << "Add ";
  if (this->_memoryTwo!=this->_memoryOne)
    descr << "Line between (" << this->_memoryOne.first << ","
	  << this->_memoryOne.second << ") and (" << this->_memoryTwo.first
	  << "," << this->_memoryTwo.second << ")";
  else
    descr << "Tadpole at (" << this->_memoryOne.first << ","
	  << this->_memoryOne.second << ")";
  this->pushHistory(descr.str());
  this->updatePostscript();
  return;
}
#define RESELECT_VERTEX {\
  set<vertex>::iterator found=\
    this->_status.vertices.find(vertex(this->_memoryOne));\
  this->_alpha.vertices.insert(*found);\
  this->_status.vertices.erase(found);\
  for (multiset<line>::iterator\
	 i=this->_status.lines.begin(),j;i!=this->_status.lines.end();)\
    if (i->vOne==this->_memoryOne||i->vTwo==this->_memoryOne) {\
      line newLine(*i);\
      if (i->vOne==this->_memoryOne)\
	newLine.selOne=true;\
      if (i->vTwo==this->_memoryOne)\
	newLine.selTwo=true;\
      this->_alpha.lines.insert(newLine);\
      this->_status.lines.erase(j=i++);\
    } else\
      ++i;\
  this->updateVertexList();\
  this->updateLineList();\
}
//
void QFeynCADEditor::deleteVertex() {
  if (!this->_alpha.vertices.size())
    RESELECT_VERTEX
  if (this->_alpha.lines.size())
    switch (QMessageBox::information(this,"QFeynCADEditor",
				     "You selected for deletion a vertex "
				     "that is connected to lines.\n"
				     "If you go on with this operation, "
				     "you will lose forever such lines.",
				     "I don't care: Delete lines too",
				     "Don't delete vertex",
				     QString::null,0,1)) {
    case 0:
      break;
    default:
      this->mergeAlphaUniquifying();
      this->repaint();
      return;
      break;
    }
  this->_alpha.clear();
  this->repaint();
  ostringstream descr;
  descr << "Remove Vertex at (" << this->_memoryOne.first << ","
	<< this->_memoryOne.second << ")";
  this->pushHistory(descr.str());
  this->updatePostscript();
  return;
}
#define RESELECT_LINES {\
  pair<multiset<line>::iterator,multiset<line>::iterator>\
    range=this->_status.lines.\
    equal_range(PROTO_LINE_OR_TADPOLE(this->_memoryOne,this->_memoryTwo));\
  for (multiset<line>::iterator i=range.first,j;i!=range.second;) {\
    line newLine(*i);\
    this->_alpha.lines.insert(newLine);\
    this->_status.lines.erase(j=i++);\
  }\
  /* In questo caso non e' necessario escludere esplicitamente i tadpoli */\
  /*  (tanto arrivati a questo punto non ci sono piu' linee comunque) */\
  range=this->_status.lines.\
    equal_range(PROTO_LINE_OR_TADPOLE(this->_memoryTwo,this->_memoryOne));\
  for (multiset<line>::iterator i=range.first,j;i!=range.second;) {\
    line newLine(*i);\
    this->_alpha.lines.insert(newLine);\
    this->_status.lines.erase(j=i++);\
  }\
  this->updateLineList();\
}
//
void QFeynCADEditor::deleteLines() {
  if (!this->_alpha.lines.size())
    RESELECT_LINES
  this->_alpha.clear();
  this->repaint();
  ostringstream descr;
  descr << "Remove ";
  if (this->_memoryTwo!=this->_memoryOne)
    descr << "Lines between (" << this->_memoryOne.first << ","
	  << this->_memoryOne.second << ") and (" << this->_memoryTwo.first
	  << "," << this->_memoryTwo.second << ")";
  else
    descr << "Tadpoles at (" << this->_memoryOne.first << ","
	  << this->_memoryOne.second << ")";
  this->pushHistory(descr.str());
  this->updatePostscript();
  return;
}
#define RESELECT_TAG {\
  set<tag>::iterator found=this->_status.tags.find(tag(this->_memoryOne));\
  this->_alpha.tags.insert(*found);\
  this->_status.tags.erase(found);\
  this->updateTagList();\
}
//
void QFeynCADEditor::deleteTag() {
  if (!this->_alpha.tags.size())
    RESELECT_TAG
  this->_alpha.clear();
  this->repaint();
  ostringstream descr;
  descr << "Remove Tag at (" << this->_memoryOne.first << ","
	<< this->_memoryOne.second << ")";
  this->pushHistory(descr.str());
  this->updatePostscript();
  return;
}

void QFeynCADEditor::prepareModifyVertex() {
  // Bisogna rievidenziare il vertice, e le linee a lui collegate
  RESELECT_VERTEX
  //
  emit setMainAppEnabledReq(false);
  this->_modifyVertex->setEnabled(true);
  ostringstream descr;
  descr << "Modifying Vertex at (" << this->_memoryOne.first << ","
	<< this->_memoryOne.second << ")...";
  emit message(true,descr.str());
  this->repaint();
  this->_modifyVertex->PushButtonApply->setEnabled(false);
  this->_modifyVertex->show();
  this->_runawayDialogCatcher->start(100);
  return;
}
#undef RESELECT_VERTEX
void QFeynCADEditor::prepareModifyLines() {
  // Bisogna rievidenziare le linee
  RESELECT_LINES
  //
  emit setMainAppEnabledReq(false);
  this->_modifyLineParent->setEnabled(true);
  ostringstream descr;
  descr << "Modifying ";
  if (this->_memoryTwo!=this->_memoryOne)
    descr << "Lines between (" << this->_memoryOne.first << ","
	  << this->_memoryOne.second << ") and (" << this->_memoryTwo.first
	  << "," << this->_memoryTwo.second << ")";
  else
    descr << "Tadpoles at (" << this->_memoryOne.first << ","
	  << this->_memoryOne.second << ")";
  descr << "...";
  emit message(true,descr.str());
  this->repaint();
  this->_modifyLineParent->deactivateApply();
  this->_modifyLineParent->show();
  this->_runawayDialogCatcher->start(100);
  return;
}
#undef RESELECT_LINES
void QFeynCADEditor::prepareModifyTag() {
  // Bisogna rievidenziare la tag
  RESELECT_TAG
  //
  emit setMainAppEnabledReq(false);
  this->_modifyTag->setEnabled(true);
  ostringstream descr;
  descr << "Modifying Tag at (" << this->_memoryOne.first << ","
	<< this->_memoryOne.second << ")...";
  emit message(true,descr.str());
  this->repaint();
  this->_modifyTag->PushButtonApply->setEnabled(false);
  this->_modifyTag->show();
  this->_runawayDialogCatcher->start(100);
  return;
}
#undef RESELECT_TAG

void QFeynCADEditor::modifyVertex() {
  this->_modifyVertex->PushButtonApply->setEnabled(false);
  this->_modifyVertex->PushButtonClose->setEnabled(false);
  vertex newVertex=this->_modifyVertex->read();
  QFeynCADEditor::_vertex=newVertex;
  // Se necessario si cancella il vertice
  if (this->_modifyVertex->isDeleted()) {
    if (this->_alpha.lines.size())
      switch (QMessageBox::information(this,"QFeynCADEditor",
				       "You selected for deletion a vertex "
				       "that is connected to lines.\n"
				       "If you go on with this operation, "
				       "you will lose forever such lines.",
				       "I don't care: Delete lines too",
				       "Don't delete vertex",
				       QString::null,0,1)) {
      case 1:
	this->repaint();
	this->_modifyVertex->PushButtonClose->setEnabled(true);
	this->_modifyVertex->show();
	this->_runawayDialogCatcher->start(100);
	return;
	break;
      default: break;
      }
    this->_alpha.lines.clear();
    this->_alpha.vertices.clear();
    this->updateLineList();
    this->updateVertexList();
    this->repaint();
    this->updatePostscript();
    this->_modifyVertex->PushButtonClose->setEnabled(true);
    this->_modifyVertex->show();
    this->_runawayDialogCatcher->start(100);
    return;
  } else if (this->_status.vertices.find(newVertex)!=
	     this->_status.vertices.end()) {
    ostringstream message;
    message << "A different vertex is already present at position ("
            << newVertex.pos.first << "," << newVertex.pos.second << ")."
            << endl << "Please choose another placement.";
    QMessageBox::warning(this,"QFeynCADEditor",message.str().c_str());
    this->repaint();
    this->_modifyVertex->PushButtonClose->setEnabled(true);
    this->_modifyVertex->show();
    this->_runawayDialogCatcher->start(100);
    return;
  }
  // Si calcola il vettore di traslazione
  int xTransl=newVertex.pos.first-this->_alpha.vertices.begin()->pos.first,
    yTransl=newVertex.pos.second-this->_alpha.vertices.begin()->pos.second;
  // Si trasla l'(_alpha) di conseguenza
  this->translateAlpha(xTransl,yTransl);
  // Si sostituisce il nuovo vertice
  this->_alpha.vertices.clear();
  this->_alpha.vertices.insert(newVertex);
  // Bleah
  this->updateVertexList();
  this->repaint();
  if (this->updatePostscript()) {
    status trueStatus=this->_status;
    // We temporarily add (_alpha) to (_status)
    this->_status.vertices.insert(*this->_alpha.vertices.begin());
    for (multiset<line>::const_iterator
	   i=this->_alpha.lines.begin();i!=this->_alpha.lines.end();++i)
      this->_status.lines.insert(*i);
    //
    this->_last=Vertex,this->_lastOne=this->_memoryOne;
    ostringstream descr;
    descr << "Modify Vertex at (" << this->_memoryOne.first << ","
	  << this->_memoryOne.second << ")";
    this->pushHistory(descr.str());
    // We come back to the original (_status)
    this->_status=trueStatus;
  }
  this->_modifyVertex->PushButtonClose->setEnabled(true);
  return;
}
void QFeynCADEditor::modifyLines() {
  this->_modifyLineParent->deactivateClose();
  QFeynCADLineEdit* currentLineEdit=
    dynamic_cast<QFeynCADLineEdit*>(this->_modifyLineParent->currentPage());
  QFeynCADTadpoleEdit* currentTadpoleEdit=
    dynamic_cast<QFeynCADTadpoleEdit*>(this->_modifyLineParent->currentPage());
  if (!currentLineEdit&&!currentTadpoleEdit)
    abort(); // Agonizing pain
  if (currentLineEdit) {
    if (!currentLineEdit->isDeleted()) {
      QFeynCADEditor::_line=currentLineEdit->read();
      QFeynCADEditor::_line.angleOne=-361;
      QFeynCADEditor::_line.angleTwo=-361;
      //QFeynCADEditor::_line.tension=1;
    }
  } else
    if (!currentTadpoleEdit->isDeleted()) {
      QFeynCADEditor::_tadpole=currentTadpoleEdit->read();
      //QFeynCADEditor::_tadpole.angleOne=90;
      //QFeynCADEditor::_tadpole.angleTwo=90;
      //QFeynCADEditor::_tadpole.tension=4;
    }
  // Si rimuovono tutte le linee evidenziate
  this->_alpha.lines.clear();
  // Si aggiungono le linee modificate presenti nei dialoghi
  for (register unsigned i=0;i<this->_modifyLine.size();++i)
    if (this->_modifyLine[i].first) {
      if (!this->_modifyLine[i].first->isDeleted())
	this->_alpha.lines.insert(this->_modifyLine[i].first->read());
    } else
      if (!this->_modifyLine[i].second->isDeleted())
	this->_alpha.lines.insert(this->_modifyLine[i].second->read());
  // Bleah
  this->updateLineList();
  this->repaint();
  if (this->updatePostscript()) {
    status trueStatus=this->_status;
    // We temporarily add (_alpha) to (_status)
    for (multiset<line>::const_iterator
	   i=this->_alpha.lines.begin();i!=this->_alpha.lines.end();++i)
      this->_status.lines.insert(*i);
    //
    this->_last=Line,
      this->_lastOne=this->_memoryOne,this->_lastTwo=this->_memoryTwo;
    ostringstream descr;
    descr << "Modify ";
    if (this->_memoryTwo!=this->_memoryOne)
      descr << "Lines between (" << this->_memoryOne.first << ","
	    << this->_memoryOne.second << ") and (" << this->_memoryTwo.first
	    << "," << this->_memoryTwo.second << ")";
    else
      descr << "Tadpoles at (" << this->_memoryOne.first << ","
	    << this->_memoryOne.second << ")";
    this->pushHistory(descr.str());
    // We come back to the original (_status)
    this->_status=trueStatus;
  }
  this->_modifyLineParent->activateClose();
  return;
}
void QFeynCADEditor::modifyTag() {
  this->_modifyTag->PushButtonApply->setEnabled(false);
  this->_modifyTag->PushButtonClose->setEnabled(false);
  tag newTag=this->_modifyTag->read();
  QFeynCADEditor::_tag=newTag;
  // Se necessario si cancella la tag
  if (this->_modifyTag->isDeleted()) {
    this->_alpha.tags.clear();
    this->updateTagList();
    this->repaint();
    this->updatePostscript();
    this->_modifyTag->PushButtonClose->setEnabled(true);
    this->_modifyTag->show();
    this->_runawayDialogCatcher->start(100);
    return;
  } else if (this->_status.tags.find(newTag)!=this->_status.tags.end()) {
    ostringstream message;
    message << "A different tag is already present at position ("
            << newTag.pos.first << "," << newTag.pos.second << ")."
            << endl << "Please choose another placement.";
    QMessageBox::warning(this,"QFeynCADEditor",message.str().c_str());
    this->repaint();
    this->_modifyTag->PushButtonClose->setEnabled(true);
    this->_modifyTag->show();
    this->_runawayDialogCatcher->start(100);
    return;
  }
  // Si sostituisce la nuova tag
  this->_alpha.tags.clear();
  this->_alpha.tags.insert(newTag);
  // Bleah
  this->updateTagList();
  this->repaint();
  if (this->updatePostscript()) {
    status trueStatus=this->_status;
    // We temporarily add (_alpha) to (_status)
    this->_status.tags.insert(*this->_alpha.tags.begin());
    //
    this->_last=Tag,this->_lastOne=this->_memoryOne;
    ostringstream descr;
    descr << "Modify Tag at (" << this->_memoryOne.first << ","
	  << this->_memoryOne.second << ")";
    this->pushHistory(descr.str());
    // We come back to the original (_status)
    this->_status=trueStatus;
  }
  this->_modifyTag->PushButtonClose->setEnabled(true);
  return;
}
void QFeynCADEditor::editLast() {
  // Non devono esserci spawning di editor multipli
  if (this->_mode==DrawMode&&this->isEnabled())
    switch (this->_last) {
    case Nothing: break;
    case Vertex: {
      this->_memoryOne=this->_lastOne;
      set<vertex>::iterator i=
	this->_status.vertices.find(vertex(this->_memoryOne));
      if (i!=this->_status.vertices.end()) {
	this->_modifyVertex->setTo(*i);
	this->prepareModifyVertex();
      } else {
	ostringstream text;
	text << "No vertices left at (" << this->_memoryOne.first << ","
	     << this->_memoryOne.second << ").";
	emit message(false,text.str().c_str());
      }
      break;
    }
    case Line: {
      this->_memoryOne=this->_lastOne;
      this->_memoryTwo=this->_lastTwo;
      REBUILD_MODIFY_LINE_PARENT
      pair<multiset<line>::const_iterator,multiset<line>::const_iterator>
	range=this->_status.lines.
	equal_range(PROTO_LINE_OR_TADPOLE(this->_memoryOne,this->_memoryTwo));
      register unsigned count=0;
      for (multiset<line>::const_iterator
	     i=range.first;i!=range.second;++i,++count)
	REBUILD_MODIFY_LINE_ITEM(i,count)
      // Qui bisogna escludere esplicitamente i tadpoli,
      //  altrimenti le linee vengono prese due volte
      if (this->_memoryOne!=this->_memoryTwo) {
	range=this->_status.lines.
	  equal_range(PROTO_LINE_OR_TADPOLE(this->_memoryTwo,
					    this->_memoryOne));
	for (multiset<line>::const_iterator
	       i=range.first;i!=range.second;++i,++count)
	  REBUILD_MODIFY_LINE_ITEM(i,count)
      }
      if (count)
	this->prepareModifyLines();
      else {
	ostringstream text;
	text << "No ";
	if (this->_memoryTwo!=this->_memoryOne)
	  text << "lines left between (" << this->_memoryOne.first << ","
	       << this->_memoryOne.second << ") and ("
	       << this->_memoryTwo.first << "," << this->_memoryTwo.second
	       << ").";
	else
	  text << "tadpoles left at (" << this->_memoryOne.first << ","
	       << this->_memoryOne.second << ").";
	emit message(false,text.str().c_str());
      }
      break;
    }
#undef REBUILD_MODIFY_LINE_PARENT
#undef REBUILD_MODIFY_LINE_ITEM
    case Tag: {
      this->_memoryOne=this->_lastOne;
      set<tag>::iterator i=this->_status.tags.find(tag(this->_memoryOne));
      if (i!=this->_status.tags.end()) {
	this->_modifyTag->setTo(*i);
	this->prepareModifyTag();
      } else {
	ostringstream text;
	text << "No tags left at (" << this->_memoryOne.first << ","
	     << this->_memoryOne.second << ").";
	emit message(false,text.str().c_str());
      }
      break;
    }
    }
  return;
}

void QFeynCADEditor::acceptPaste() {
  this->setModeTo(SelectMode);
  this->updatePostscript();
  this->pushHistory("Paste Selection");
  return;
}
void QFeynCADEditor::abandonPaste() {
  this->_alpha.clear();
  this->setModeTo(SelectMode);
  return;
}
void QFeynCADEditor::deleteSelection() {
  this->_alpha.clear();
  ostringstream descr;
  descr << "Delete Selection (" << this->_selRectOne.first << ","
	<< this->_selRectOne.second << ")<>(" << this->_selRectTwo.first << ","
	<< this->_selRectTwo.second << ")";
  this->pushHistory(descr.str());
  //
  this->_selRectOne=make_pair(-1,-1);
  this->_selRectTwo=make_pair(-1,-1);
  this->repaint();
  this->updatePostscript();
  return;
}
void QFeynCADEditor::clipToSelection() {
  this->stopRunawayDialogCatcher();
  this->_modifySelection->close();
  pair<int,int> selRectOne=this->_selRectOne,selRectTwo=this->_selRectTwo;
  // Si cancella il rettangolo di selezione
  this->_selRectOne=make_pair(-1,-1);
  this->_selRectTwo=make_pair(-1,-1);
  // Si riportano giu' gli oggetti
  this->mergeAlphaUniquifying();
  // Si resetta (_modifySelection) [effetti di bordo!]
  this->_modifySelection->newBoundariesAre(-1,-1,-1,-1);
  // Si calcola il vettore di traslazione per gli oggetti
  int xObjTransl=-min(selRectOne.first,selRectTwo.first),
    yObjTransl=-min(selRectOne.second,selRectTwo.second);
  // Si traslano tutti gli oggetti
  set<vertex> newVertices;
  for (set<vertex>::iterator i=this->_status.vertices.begin();
       i!=this->_status.vertices.end();++i) {
    vertex newVertex(*i);
    newVertex.pos.first+=xObjTransl;
    newVertex.pos.second+=yObjTransl;
    newVertices.insert(newVertex);
  }
  this->_status.vertices=newVertices;
  multiset<line> newLines;
  for (multiset<line>::iterator
	 i=this->_status.lines.begin();i!=this->_status.lines.end();++i) {
    line newLine(*i);
    newLine.vOne.first+=xObjTransl;
    newLine.vOne.second+=yObjTransl;
    newLine.vTwo.first+=xObjTransl;
    newLine.vTwo.second+=yObjTransl;
    newLines.insert(newLine);
  }
  this->_status.lines=newLines;
  set<tag> newTags;
  for (set<tag>::iterator
	 i=this->_status.tags.begin();i!=this->_status.tags.end();++i) {
    tag newTag(*i);
    newTag.pos.first+=xObjTransl;
    newTag.pos.second+=yObjTransl;
    newTags.insert(newTag);
  }
  this->_status.tags=newTags;
  //
  ostringstream descr;
  descr << "Clipping to Selection (" << selRectOne.first << ","
	<< selRectOne.second << ")<>(" << selRectTwo.first << ","
	<< selRectTwo.second << ")";
  // Si ridimensiona la griglia
  this->setGridSizeTo(abs(selRectTwo.first-selRectOne.first),
		      abs(selRectTwo.second-selRectOne.second));
  this->pushHistory(descr.str());
  this->repaint();
  return;
}
void QFeynCADEditor::rotateAlpha(int x,int y,int angle) {
  this->_newSelRectRotationCenter=make_pair(x,y);
  this->_newSelRectRotationAngle=double(angle);
  if (this->_modifySelection->_alpha.size())
    this->_alpha=this->_modifySelection->_transformedAlpha;
  this->repaint();
  return;
}
void QFeynCADEditor::acceptRotation() {
  this->stopRunawayDialogCatcher();
  this->_modifySelection->close();
  // Questa prima pulizia ci vuole a causa degli effetti di bordo che verranno
  //  Si cancella il rettangolo di selezione
  this->_selRectOne=make_pair(-1,-1);
  this->_selRectTwo=make_pair(-1,-1);
  //  Si cancella la cornice di selezione
  this->_newSelRectOne=make_pair(-1,-1);
  this->_newSelRectTwo=make_pair(-1,-1);
  this->_newSelRectRotationCenter=make_pair(-1,-1);
  this->_newSelRectRotationAngle=double(0);
  //
  this->mergeAlphaUniquifying();
  pair<int,int> memOne=this->_selRectOne,memTwo=this->_selRectTwo;
  // Si resetta (_modifySelection) [effetti di bordo!]
  this->_modifySelection->newBoundariesAre(-1,-1,-1,-1);
  // Ora si cancella definitivamente
  //  Si cancella il rettangolo di selezione
  this->_selRectOne=make_pair(-1,-1);
  this->_selRectTwo=make_pair(-1,-1);
  //  Si cancella la cornice di selezione
  this->_newSelRectOne=make_pair(-1,-1);
  this->_newSelRectTwo=make_pair(-1,-1);
  this->_newSelRectRotationCenter=make_pair(-1,-1);
  this->_newSelRectRotationAngle=double(0);
  //
  this->repaint();
  //
  emit setMainAppEnabledReq(true);
  emit message(false,"");
  emit newSelection();
  //
  this->updateVertexList();
  this->updateLineList();
  this->updateTagList();
  //
  ostringstream descr;
  descr << "Rotate Selection (" << memOne.first << "," << memOne.second
	<< ")<>(" << memTwo.first << "," << memTwo.second << ")";
  this->pushHistory(descr.str());
  this->updatePostscript();
  return;
}
void QFeynCADEditor::abandonTransformation() {
  this->stopRunawayDialogCatcher();
  this->_modifySelection->close();
  // Ci si riappropria dei contenuti di (_alpha)
  this->_alpha=this->_modifySelection->_alpha;
  // Si resetta (_modifySelection) [effetti di bordo!]
  this->_modifySelection->newBoundariesAre(-1,-1,-1,-1);
  // Si cancella la cornice di selezione
  this->_newSelRectOne=make_pair(-1,-1);
  this->_newSelRectTwo=make_pair(-1,-1);
  this->_newSelRectRotationCenter=make_pair(-1,-1);
  this->_newSelRectRotationAngle=double(0);
  //
  this->repaint();
  //
  emit setMainAppEnabledReq(true);
  emit message(false,"");
  //
  this->updateVertexList();
  this->updateLineList();
  this->updateTagList();
  return;
}

void QFeynCADEditor::reenableApp() {
  // Se tutti i dialoghi sono chiusi si riabilita l'applicazione
  if (!(this->_modifyVertex->isShown()||
	(this->_modifyLineParent&&this->_modifyLineParent->isShown())||
	this->_modifyTag->isShown()||
	this->_modifySelection->isShown())) {
    // Si spegne il watchdog
    this->_runawayDialogCatcher->stop();
    //
    emit setMainAppEnabledReq(true);
    emit message(false,"");
    //
    if (this->_modifySelection->_alpha.size())
      this->abandonTransformation();
    else {
      // Si cancella il rettangolo di selezione
      this->_selRectOne=make_pair(-1,-1);
      this->_selRectTwo=make_pair(-1,-1);
      // Si cancella la cornice di selezione
      this->_newSelRectOne=make_pair(-1,-1);
      this->_newSelRectTwo=make_pair(-1,-1);
      this->_newSelRectRotationCenter=make_pair(-1,-1);
      this->_newSelRectRotationAngle=double(0);
      //
      this->mergeAlphaUniquifying();
      this->repaint();
      //
      this->updateVertexList();
      this->updateLineList();
      this->updateTagList();
    }
    /*
    //
    QMessageBox::warning(this,"QFeynCADEditor",
			 "Killing object editors directly should be avoided.\n"
			 "Please use provided buttons next time.","Ok");
    */
  }
  return;
}

void QFeynCADEditor::translateAlpha(int xTransl,int yTransl) {
  // Si traslano gli oggetti selezionati
  set<vertex> newVertices;
  for (set<vertex>::iterator i=this->_alpha.vertices.begin();
       i!=this->_alpha.vertices.end();++i) {
    vertex newVertex(*i);
    newVertex.pos.first+=xTransl;
    newVertex.pos.second+=yTransl;
    newVertices.insert(newVertex);
  }
  this->_alpha.vertices=newVertices;
  multiset<line> newLines;
  for (multiset<line>::iterator
	 i=this->_alpha.lines.begin();i!=this->_alpha.lines.end();++i) {
    line newLine(*i);
    if (newLine.selOne)
      { newLine.vOne.first+=xTransl; newLine.vOne.second+=yTransl; }
    if (newLine.selTwo)
      { newLine.vTwo.first+=xTransl; newLine.vTwo.second+=yTransl; }
    newLines.insert(newLine);
  }
  this->_alpha.lines=newLines;
  set<tag> newTags;
  for (set<tag>::iterator
	 i=this->_alpha.tags.begin();i!=this->_alpha.tags.end();++i) {
    tag newTag(*i);
    newTag.pos.first+=xTransl;
    newTag.pos.second+=yTransl;
    newTags.insert(newTag);
  }
  this->_alpha.tags=newTags;
  return;
}
bool QFeynCADEditor::mergeAlphaUniquifying() {
  emit setMainAppEnabledReq(true);
  // We merge the objects present in (alpha) into the existing ones,
  //  avoiding collisions
  unsigned counter=0;
  ostringstream collisions;
  //  We create a catalogue of vertices, to be able to move lines after
  map<pair<int,int>,pair<int,int> > vertexCatalogue;
  //
  for (set<vertex>::const_iterator
	 i=this->_alpha.vertices.begin(),j;i!=this->_alpha.vertices.end();++i)
    if ((j=this->_status.vertices.find(*i))!=this->_status.vertices.end()) {
      vertex newVertex(*i);
      ++counter;
      collisions << "[#" << counter << "#] Vertex at ("
		 << newVertex.pos.first << "," << newVertex.pos.second
		 << ") was moved to (";
      register unsigned distance=1;
      bool isOk=false;
      // Remember: (*j) stays fixed
      while (1) {
	newVertex.pos.first=j->pos.first+distance;
	for (register unsigned k=0;k<distance+1;++k) {
	  newVertex.pos.second=-newVertex.pos.first+
	    j->pos.first+j->pos.second+distance;
	  if (this->_status.vertices.find(newVertex)==
	      this->_status.vertices.end()) {
	    this->_status.vertices.insert(newVertex);
	    vertexCatalogue.insert(make_pair(j->pos,newVertex.pos));
	    collisions << newVertex.pos.first << "," << newVertex.pos.second
		       << ")." << endl;
	    isOk=true;
	    break;
	  }
	  --newVertex.pos.first;
	}
	if (isOk)
	  break;
	++distance;
      }
    } else {
      this->_status.vertices.insert(*i);
      vertexCatalogue.insert(make_pair(i->pos,i->pos));
    }
  //  Now we have to move lines too
  //   Solo i vertici che appartengono all'(_alpha) devono essere mossi
  for (multiset<line>::const_iterator
	 i=this->_alpha.lines.begin();i!=this->_alpha.lines.end();++i) {
    line newLine(*i);
    if (newLine.selOne)
      newLine.vOne=vertexCatalogue[newLine.vOne];
    if (newLine.selTwo)
      newLine.vTwo=vertexCatalogue[newLine.vTwo];
    //   Qui bisogna anche deselezionare i vertici!
    newLine.selOne=false;
    newLine.selTwo=false;
    this->_status.lines.insert(newLine);
  }
  //  Tags must be checked too, instead
  for (set<tag>::const_iterator
	 i=this->_alpha.tags.begin(),j;i!=this->_alpha.tags.end();++i)
    if ((j=this->_status.tags.find(*i))!=this->_status.tags.end()) {
      tag newTag(*i);
      ++counter;
      collisions << "[#" << counter << "#] Tag at (" << newTag.pos.first
		 << "," << newTag.pos.second << ") was moved to (";
      register unsigned distance=1;
      bool isOk=false;
      // Remember: (*j) stays fixed
      while (1) {
	newTag.pos.first=j->pos.first+distance;
	for (register unsigned k=0;k<distance+1;++k) {
	  newTag.pos.second=-newTag.pos.first+
	    j->pos.first+j->pos.second+distance;
	  if (this->_status.tags.find(newTag)==this->_status.tags.end()) {
	    this->_status.tags.insert(newTag);
	    collisions << newTag.pos.first << "," << newTag.pos.second
		       << ")." << endl;
	    isOk=true;
	    break;
	  }
	  --newTag.pos.first;
	}
	if (isOk)
	  break;
	++distance;
      }
    } else
      this->_status.tags.insert(*i);
  //
  this->_alpha.clear();
  //
  if (counter) {
    ostringstream complete;
    complete << "Stacked vertices/tags were found during a merge operation."
	     << endl << "The following operation"
	     << (counter==1?" was":"s were")
	     << " necessary to solve the conflict"
	     << (counter==1?"":"s") << ":" << endl << collisions.str();
    QMessageBox::information(this,"QFeynCADEditor",complete.str().c_str(),
			     "Accept");
    //
    ostringstream descr;
    descr << "Uniquify";
    this->pushHistory(descr.str());
    // In questo caso la situazione e' cambiata
    this->updatePostscript();
  }
  this->repaint();
  this->updateVertexList();
  this->updateLineList();
  this->updateTagList();
  return counter;
}

bool QFeynCADEditor::updatePostscript() {
  if (this->_fileName=="")
    return false;
  // Si cambia directory ogni volta per permettere un eventuale recovery
  if (chdir(QFeynCADEditor::_tmpPath.c_str())) {
    if (!this->_inBatchMode) {
      ostringstream message;
      message << "Could not access selected temporary directory" << endl
	      << "\"" << QFeynCADEditor::_tmpPath << "\"." << endl
	      << "Please check carefully its name and permissions." << endl;
      QMessageBox::warning(0,"QFeynCADEditor",message.str().c_str());
    }
    return false;
  }
  //
  ostringstream mpName;
  mpName << this->_fileName << ".mp";
  ofstream mp(mpName.str().c_str());
  mp << "% MetaPost file generated by FeynCAD v1 - (c) 1999-2004 Paolo Ribeca"
     << endl << "input " << QFeynCADEditor::_macrosPath << endl;
  if (QFeynCADEditor::_addendumPath!=string(""))
    mp << "input "
       << QFeynCADEditor::_addendumPath << endl;
  istringstream settings(QFeynCADEditor::_metapostSettings);
  int unit;
  float value;
#define READ_METAPOST_PREAMBLE(property) \
  settings >> unit;\
  if (unit!=-1) {\
    settings >> value;\
    mp << property << ":=" << value;\
    switch (unit) {\
    case 0: mp << "mm"; break;\
    case 1: mp << "in"; break;\
    case 2: mp << "pt"; break;\
    case 3: mp << "dd"; break;\
    case 4: mp << "bp"; break;\
    default: break;\
    }\
    mp << ";" << endl;\
  }
  //
  READ_METAPOST_PREAMBLE("thin")
  READ_METAPOST_PREAMBLE("arrow_len")
  READ_METAPOST_PREAMBLE("curly_len")
  READ_METAPOST_PREAMBLE("dash_len")
  READ_METAPOST_PREAMBLE("dot_len")
  READ_METAPOST_PREAMBLE("wiggly_len")
  READ_METAPOST_PREAMBLE("zigzag_len")
  READ_METAPOST_PREAMBLE("zigzag_width")
#undef READ_METAPOST_PREAMBLE
  mp << "arrow_ang:=" << (settings>>unit,unit) << ";" << endl;
  mp << "dbl_arrow_multiplier:=" << (settings>>unit,unit) << ";" << endl;
  mp << "wiggly_slope:=" << (settings>>unit,unit) << ";" << endl;
  mp << "beginchar(1," << this->_status.gridSize.first << UNIT << ","
     << this->_status.gridSize.second << UNIT << ",0);" << endl;
  multimap<unsigned,string,greater<unsigned> > elements;
  for (multiset<line>::iterator
	 i=this->_status.lines.begin();i!=this->_status.lines.end();++i)
    elements.insert(make_pair(i->depth,i->toMetapost()));
  for (multiset<line>::iterator
	 i=this->_alpha.lines.begin();i!=this->_alpha.lines.end();++i)
    elements.insert(make_pair(i->depth,i->toMetapost()));
  for (set<vertex>::iterator
	 i=this->_status.vertices.begin();i!=this->_status.vertices.end();++i)
    elements.insert(make_pair(i->depth,i->toMetapost()));
  for (set<vertex>::iterator
	 i=this->_alpha.vertices.begin();i!=this->_alpha.vertices.end();++i)
    elements.insert(make_pair(i->depth,i->toMetapost()));
  for (set<tag>::iterator
	 i=this->_status.tags.begin();i!=this->_status.tags.end();++i)
    elements.insert(make_pair(i->depth,i->toMetapost()));
  for (set<tag>::iterator
	 i=this->_alpha.tags.begin();i!=this->_alpha.tags.end();++i)
    elements.insert(make_pair(i->depth,i->toMetapost()));
  for (multimap<unsigned,string>::iterator
	 i=elements.begin();i!=elements.end();++i)
    mp << i->second;
  mp << "endchar;" << endl << "end." << endl << "endinput;" << endl;
  mp.close();
  ostringstream texName;
  texName << this->_fileName << ".tex";
  ofstream tex(texName.str().c_str());
  tex << "\\batchmode" << endl << "\\documentclass{article}" << endl
    //"\\documentclass{report}\n"
    //"\\usepackage{palatino}\n"
    //"\\usepackage[T1]{fontenc}\n"
      << "\\usepackage{graphicx}" << endl
    //"\\usepackage{pslatex}\n" // doesn't work properly for boundaries
      << "\\usepackage{amsmath}" << endl << "\\usepackage{amssymb}" << endl
      << "\\pagestyle{empty}" << endl << "\\begin{document}" << endl
      << "{\\centering\\includegraphics{" << this->_fileName << ".1}\\par}"
      << endl
    //"\\special{psfile="TMP_BASE_NAME".1}\n"
      << "\\end{document}" << endl;
  tex.close();
  ostringstream mpCommand;
  mpCommand << QFeynCADEditor::_metapostPath << " --interaction batchmode "
	    << this->_fileName << ".mp >& /dev/null 2>&1";
  if (system(mpCommand.str().c_str())) {
    if (!this->_inBatchMode) {
      ostringstream message;
      message << "Could not properly execute MetaPost\n(\"" << mpCommand.str()
	      << "\").";
      QMessageBox::warning(this,"QFeynCADEditor",message.str().c_str());
    }
    return false;
  }
  ostringstream latexCommand;
  latexCommand << QFeynCADEditor::_latexPath << " --output-comment "
	       << "\"DVI file generated by FeynCAD v1 - "
	       << "(c) 1999-2004 Paolo Ribeca\" " << this->_fileName
	       << ".tex >& /dev/null 2>&1";
  if (system(latexCommand.str().c_str())) {
    if (!this->_inBatchMode) {
      ostringstream message;
      message << "Could not properly execute LaTeX\n(\"" << latexCommand.str()
	      << "\").";
      QMessageBox::warning(this,"QFeynCADEditor",message.str().c_str());
    }
    return false;
  }
  ostringstream dvipsCommand;
  dvipsCommand << QFeynCADEditor::_dvipsPath << " -E -o " << this->_fileName
	       << ".eps " << this->_fileName << ".dvi >& /dev/null 2>&1";
  if (system(dvipsCommand.str().c_str())) {
    if (!this->_inBatchMode) {
      ostringstream message;
      message << "Could not properly execute dvips\n(\"" << dvipsCommand.str()
	      << "\").";
      QMessageBox::warning(this,"QFeynCADEditor",message.str().c_str());
    }
    return false;
  }
  ostringstream removeCommand;
  removeCommand << "rm -f " << this->_fileName << ".1 " << this->_fileName
		<< ".aux " << this->_fileName << ".dvi " << this->_fileName
		<< ".log " << this->_fileName << ".mpx ";
  if (!QFeynCADEditor::_keepFiles)
    removeCommand << this->_fileName << ".mp " << this->_fileName << ".tex ";
  removeCommand << ">& /dev/null 2>&1";
  if (system(removeCommand.str().c_str())) {
    if (!this->_inBatchMode) {
      ostringstream message;
      message << "Could not properly remove temporary files\n(\""
	      << removeCommand.str() << "\").";
      QMessageBox::warning(this,"QFeynCADEditor",message.str().c_str());
    }
    return false;
  }
  emit viewerUpdateReq(unsigned(this->width()),unsigned(this->height()));
  return true;
}

void QFeynCADEditor::dumpTo(string fcdFileName) {
  QDomDocument doc("FeynCAD");
  // Crea il nodo-documento
  QDomElement root=doc.createElement("FeynCAD");
  root.setAttribute("version","1");
  // Appende il nodo-documento all'albero
  doc.appendChild(root);
  QDomElement graph=doc.createElement("graph");
  graph.setAttribute("x",QString::number(this->_status.gridSize.first));
  graph.setAttribute("y",QString::number(this->_status.gridSize.second));
  // Attributi come data di creazione, autore, etc.
  this->_modificationDateTime=QDateTime::currentDateTime().toString();
  if (this->_creationDateTime=="")
    this->_creationDateTime=this->_modificationDateTime;
  graph.setAttribute("created",this->_creationDateTime);
  graph.setAttribute("modified",this->_modificationDateTime);
  graph.setAttribute("author",this->_author);
  graph.setAttribute("comments",this->_comments);
  //
  root.appendChild(graph);
  QDomElement vertices=doc.createElement("vertices");
  graph.appendChild(vertices);
  for (set<vertex>::iterator i=this->_status.vertices.begin();
       i!=this->_status.vertices.end();++i)
    i->toXml(doc,vertices);
  for (set<vertex>::iterator i=this->_alpha.vertices.begin();
       i!=this->_alpha.vertices.end();++i)
    i->toXml(doc,vertices);
  QDomElement lines=doc.createElement("lines");
  graph.appendChild(lines);
  for (multiset<line>::iterator
	 i=this->_status.lines.begin();i!=this->_status.lines.end();++i)
    i->toXml(doc,lines);
  for (multiset<line>::iterator
	 i=this->_alpha.lines.begin();i!=this->_alpha.lines.end();++i)
    i->toXml(doc,lines);
  QDomElement tags=doc.createElement("tags");
  graph.appendChild(tags);
  for (set<tag>::iterator
	 i=this->_status.tags.begin();i!=this->_status.tags.end();++i)
    i->toXml(doc,tags);
  for (set<tag>::iterator
	 i=this->_alpha.tags.begin();i!=this->_alpha.tags.end();++i)
    i->toXml(doc,tags);
  QString xml=doc.toString();
  ofstream outFile(fcdFileName.c_str());
  if (!outFile) {
    ostringstream message;
    message << "Could not open selected output file\n\"" << fcdFileName
	    << "\"\nfor writing.";
    QMessageBox::warning(this,"QFeynCADEditor",message.str().c_str());
    return;
  }
  outFile << xml;
  if (!this->_isUpToDate)
    { this->_isUpToDate=true; emit titleUpdateReq(); }
  return;
}
bool QFeynCADEditor::readFromFile(string fcdFileName) {
  QDomDocument doc("FeynCAD");
  QFile file(fcdFileName.c_str());
  if (!file.open(IO_ReadOnly)) {
    if (!this->_inBatchMode) {
      ostringstream message;
      message << "Could not open selected input file\n\"" << fcdFileName
	      << "\"\nfor reading.";
      QMessageBox::warning(this,"QFeynCADEditor",message.str().c_str());
    }
    return false;
  }
  if (!doc.setContent(&file)) {
    if (!this->_inBatchMode) {
      ostringstream message;
      message << "Error parsing selected input file\n\"" << fcdFileName
	      << "\":\ndocument is malformed.";
      QMessageBox::warning(this,"QFeynCADEditor",message.str().c_str());
    }
    file.close();
    return false;
  }
  file.close();
  QDomNode node=doc.firstChild();
  QDomElement root=node.toElement();
  if (root.isNull()||root.tagName()!=QString("FeynCAD")) {
    if (!this->_inBatchMode) {
      ostringstream message;
      message << "Selected input file\n\"" << fcdFileName
	      << "\"\nis not valid FeynCAD XML.";
      QMessageBox::warning(this,"QFeynCADEditor",message.str().c_str());
    }
    return false;
  }
  if (root.attribute("version")!=QString("1")) {
    if (!this->_inBatchMode) {
      ostringstream message;
      message << "Wrong FeynCAD XML version in selected input file\n\""
	      << fcdFileName << "\":\ngot \"" << root.attribute("version")
	      << "\", expected \"1\".";
      QMessageBox::warning(this,"QFeynCADEditor",message.str().c_str());
    }
    return false;
  }
  node=node.firstChild();
  QDomElement graph=node.toElement();
  bool documentIsOk=true;
  status tmp;
  // Solito viscido trucco
  while (1) {
    if (graph.isNull()||graph.tagName()!=QString("graph"))
      { documentIsOk=false; break; }
    tmp.gridSize.first=graph.attribute("x").toUInt();
    tmp.gridSize.second=graph.attribute("y").toUInt();
    if (!tmp.gridSize.first||!tmp.gridSize.second) {
      if (!this->_inBatchMode) {
	ostringstream message;
	message << "In selected input file\n\"" << fcdFileName
		<< "\":\nwrong dimensions for grid.";
	QMessageBox::warning(this,"QFeynCADEditor",message.str().c_str());
      }
      return false;
    }
    //
    this->_creationDateTime=graph.attribute("created","");
    this->_modificationDateTime=graph.attribute("modified","");
    this->_author=graph.attribute("author","");
    this->_comments=graph.attribute("comments","");
    //
    node=node.firstChild();
    while (!node.isNull()) {
      QDomElement e=node.toElement();
      if (!e.isNull())
	if (e.tagName()==QString("vertices")) {
	  QDomNode nnode=node.firstChild();
	  while (!nnode.isNull()) {
	    QDomElement ee=nnode.toElement();
	    if (!ee.isNull())
	      tmp.vertices.insert(vertex(ee));
	    else
	      documentIsOk=false;
	    if (!documentIsOk)
	      break;
	    nnode=nnode.nextSibling();
	  }
	} else if (e.tagName()==QString("lines")) {
	  QDomNode nnode=node.firstChild();
	  while (!nnode.isNull()) {
	    QDomElement ee=nnode.toElement();
	    if (!ee.isNull())
	      tmp.lines.insert(line(ee));
	    else
	      documentIsOk=false;
	    if (!documentIsOk)
	      break;
	    nnode=nnode.nextSibling();
	  }
	} else if (e.tagName()==QString("tags")) {
	  QDomNode nnode=node.firstChild();
	  while (!nnode.isNull()) {
	    QDomElement ee=nnode.toElement();
	    if (!ee.isNull())
	      tmp.tags.insert(tag(ee));
	    else
	      documentIsOk=false;
	    if (!documentIsOk)
	      break;
	    nnode=nnode.nextSibling();
	  }
	} else
	  { documentIsOk=false; break; }
      node=node.nextSibling();
    }
    // Necessario!
    break;
  }
  if (documentIsOk) {
    this->_history.clear();
    this->_current=0;
    this->_status=tmp;
    emit gridChanged(this->_status.gridSize.first,
		     this->_status.gridSize.second);
    this->setFixedSize(this->_status.gridSize.first*QFeynCADEditor::_gridStep+
		       2*QFeynCADEditor::_circleBaseSize+1,
		       this->_status.gridSize.second*QFeynCADEditor::_gridStep+
		       2*QFeynCADEditor::_circleBaseSize+1);
    this->updatePostscript();
    this->pushHistory("Base");
    this->_isUpToDate=true;
    emit titleUpdateReq();
    this->repaint();
  } else {
    this->_creationDateTime="";
    this->_modificationDateTime="";
    this->_author="";
    this->_comments="";
    if (!this->_inBatchMode) {
      ostringstream message;
      message << "Parse error in selected input file\n\""
	      << fcdFileName << "\".";
      QMessageBox::warning(this,"QFeynCADEditor",message.str().c_str());
    }
    return false;
  }
  return true;
}
void QFeynCADEditor::readFromBuffer(string fcdBuffer) {
  // A simplified version of the function before
  QDomDocument doc("FeynCAD");
  doc.setContent(QString(fcdBuffer.c_str()));
  QDomNode node=doc.firstChild();
  QDomElement root=node.toElement();
  node=node.firstChild();
  QDomElement graph=node.toElement();
  status tmp;
  // Solito viscido trucco
  while (1) {
    tmp.gridSize.first=graph.attribute("x").toUInt();
    tmp.gridSize.second=graph.attribute("y").toUInt();
    node=node.firstChild();
    while (!node.isNull()) {
      QDomElement e=node.toElement();
      if (!e.isNull())
	if (e.tagName()==QString("vertices")) {
	  QDomNode nnode=node.firstChild();
	  while (!nnode.isNull()) {
	    QDomElement ee=nnode.toElement();
	    if (!ee.isNull())
	      tmp.vertices.insert(vertex(ee));
	    nnode=nnode.nextSibling();
	  }
	} else if (e.tagName()==QString("lines")) {
	  QDomNode nnode=node.firstChild();
	  while (!nnode.isNull()) {
	    QDomElement ee=nnode.toElement();
	    if (!ee.isNull())
	      tmp.lines.insert(line(ee));
	    nnode=nnode.nextSibling();
	  }
	} else if (e.tagName()==QString("tags")) {
	  QDomNode nnode=node.firstChild();
	  while (!nnode.isNull()) {
	    QDomElement ee=nnode.toElement();
	    if (!ee.isNull())
	      tmp.tags.insert(tag(ee));
	    nnode=nnode.nextSibling();
	  }
	}
      node=node.nextSibling();
    }
    // Necessario!
    break;
  }
  this->_history.clear();
  this->_current=0;
  this->_status=tmp;
  emit gridChanged(this->_status.gridSize.first,this->_status.gridSize.second);
  this->setFixedSize(this->_status.gridSize.first*QFeynCADEditor::_gridStep+
		     2*QFeynCADEditor::_circleBaseSize+1,
		     this->_status.gridSize.second*QFeynCADEditor::_gridStep+
		     2*QFeynCADEditor::_circleBaseSize+1);
  this->updatePostscript();
  this->pushHistory("Base");
  this->_isUpToDate=true;
  emit titleUpdateReq();
  this->repaint();
  return;
}
void QFeynCADEditor::exportEpsTo(string tmpFileName,string epsFileName) {
  // Nel caso si completa il nome
  if (epsFileName.size()<4||
      epsFileName[epsFileName.size()-4]!='.'||
      epsFileName[epsFileName.size()-3]!='e'||
      epsFileName[epsFileName.size()-2]!='p'||
      epsFileName[epsFileName.size()-1]!='s')
    epsFileName+=string(".eps");
  ostringstream command;
  command << "cp " << tmpFileName << ".eps " << epsFileName;
  if (system(command.str().c_str())) {
    ostringstream message;
    message << "Could not open selected EPS file\n\"" << epsFileName
	    << "\"\nfor writing.";
    QMessageBox::warning(this,"QFeynCADEditor",message.str().c_str());
  }
  return;
}

void QFeynCADEditor::setGridSizeTo(int newGridSizeX,int newGridSizeY) {
  this->_status.gridSize=make_pair(newGridSizeX,newGridSizeY);
  emit gridChanged(this->_status.gridSize.first,this->_status.gridSize.second);
  this->setFixedSize(this->_status.gridSize.first*QFeynCADEditor::_gridStep+
		     2*QFeynCADEditor::_circleBaseSize+1,
		     this->_status.gridSize.second*QFeynCADEditor::_gridStep+
		     2*QFeynCADEditor::_circleBaseSize+1);
  this->repaint();
  // Si trasmette la variazione a (_modifySelection)
  if (this->_modifySelection)
    this->_modifySelection->
      actualBoundariesAre(0,this->_status.gridSize.first,
			  0,this->_status.gridSize.second);
  //
  /*
  ostringstream descr;
  descr << "Changing Grid Size to (" << newGridSizeX << ","
	<< newGridSizeY << ")";
  this->pushHistory(descr.str());
  */
  // Il file deve essere pronto _prima_ della richiesta di update del viewer
  this->updatePostscript();
  return;
}

void QFeynCADEditor::resetToDefaults() {
  QFeynCADEditor::_gridX=FEYNCAD_DEFAULT_GRID_X;
  QFeynCADEditor::_gridY=FEYNCAD_DEFAULT_GRID_Y;
  QFeynCADEditor::_gridStep=FEYNCAD_DEFAULT_GRID_STEP;
  QFeynCADEditor::_squareBaseSize=FEYNCAD_DEFAULT_SQUARE_BASE_SIZE;
  QFeynCADEditor::_circleBaseSize=FEYNCAD_DEFAULT_CIRCLE_BASE_SIZE;
  QFeynCADEditor::_markerStep=FEYNCAD_DEFAULT_MARKER_STEP;
  QFeynCADEditor::_macrosPath=FEYNCAD_DEFAULT_MACROS_PATH;
  QFeynCADEditor::_addendumPath=FEYNCAD_DEFAULT_ADDENDUM_PATH;
  QFeynCADEditor::_metapostPath=FEYNCAD_DEFAULT_METAPOST_PATH;
  QFeynCADEditor::_latexPath=FEYNCAD_DEFAULT_LATEX_PATH;
  QFeynCADEditor::_dvipsPath=FEYNCAD_DEFAULT_DVIPS_PATH;
  QFeynCADEditor::_gsPath=FEYNCAD_DEFAULT_GS_PATH;
  QFeynCADEditor::_browserPath=FEYNCAD_DEFAULT_BROWSER_PATH;
  QFeynCADEditor::_tmpPath=FEYNCAD_DEFAULT_TMP_PATH;
  QFeynCADEditor::_timeout=FEYNCAD_DEFAULT_TIMEOUT;
  QFeynCADEditor::_keepFiles=FEYNCAD_DEFAULT_KEEP_FILES;
  QFeynCADEditor::_historyDepth=FEYNCAD_DEFAULT_HISTORY_DEPTH;
  QFeynCADEditor::_metapostSettings=FEYNCAD_DEFAULT_METAPOST_SETTINGS;
  return;
}
void QFeynCADEditor::dumpConfigTo(string configFileName) {
  QDomDocument doc("FeynCADrc");
  // Crea il nodo-documento
  QDomElement root=doc.createElement("FeynCADrc");
  root.setAttribute("version","1");
  // Appende il nodo-documento all'albero
  doc.appendChild(root);
  QDomElement preferences=doc.createElement("preferences");
  root.appendChild(preferences);
  QDomElement gridSize=doc.createElement("gridSize");
  gridSize.setAttribute("x",QString::number(QFeynCADEditor::_gridX));
  gridSize.setAttribute("y",QString::number(QFeynCADEditor::_gridY));
  preferences.appendChild(gridSize);
  QDomElement editor=doc.createElement("editor");
  editor.setAttribute("gridStep",QString::number(QFeynCADEditor::_gridStep));
  editor.setAttribute("squareBaseSize",
		      QString::number(QFeynCADEditor::_squareBaseSize));
  editor.setAttribute("circleBaseSize",
		      QString::number(QFeynCADEditor::_circleBaseSize));
  editor.setAttribute("markerStep",
		      QString::number(QFeynCADEditor::_markerStep));
  preferences.appendChild(editor);
  QDomElement paths=doc.createElement("paths");
  paths.setAttribute("macros",QString(QFeynCADEditor::_macrosPath.c_str()));
  paths.setAttribute("addendum",
		     QString(QFeynCADEditor::_addendumPath.c_str()));
  paths.setAttribute("metapost",
		     QString(QFeynCADEditor::_metapostPath.c_str()));
  paths.setAttribute("latex",QString(QFeynCADEditor::_latexPath.c_str()));
  paths.setAttribute("dvips",QString(QFeynCADEditor::_dvipsPath.c_str()));
  paths.setAttribute("gs",QString(QFeynCADEditor::_gsPath.c_str()));
  paths.setAttribute("browser",QString(QFeynCADEditor::_browserPath.c_str()));
  paths.setAttribute("tmp",QString(QFeynCADEditor::_tmpPath.c_str()));
  preferences.appendChild(paths);
  QDomElement misc=doc.createElement("miscellaneous");
  misc.setAttribute("timeout",QString::number(QFeynCADEditor::_timeout));
  misc.setAttribute("keepIntermediateFiles",
		    QString::number((int)QFeynCADEditor::_keepFiles));
  misc.setAttribute("historyDepth",
		    QString::number(QFeynCADEditor::_historyDepth));
  misc.setAttribute("metapostSettings",
		    QFeynCADEditor::_metapostSettings.c_str());
  preferences.appendChild(misc);
  //
  QString xml=doc.toString();
  ofstream outFile(configFileName.c_str());
  if (!outFile) {
    ostringstream message;
    message << "Could not open selected configuration file\n\""
	    << configFileName << "\"\nfor writing.";
    QMessageBox::warning(0,"QFeynCADEditor",message.str().c_str());
    return;
  }
  outFile << xml;
  return;
}
bool QFeynCADEditor::readConfigFrom(string configFileName) {
  QDomDocument doc("FeynCADrc");
  QFile file(configFileName.c_str());
  if (!file.open(IO_ReadOnly)) {
    ostringstream message;
    message << "Could not open selected configuration file\n\""
	    << configFileName << "\"\nfor reading.";
    QMessageBox::warning(0,"QFeynCADEditor",message.str().c_str());
    return false;
  }
  if (!doc.setContent(&file)) {
    ostringstream message;
    message << "Error parsing selected configuration file\n\""
	    << configFileName << "\":\ndocument is malformed.";
    QMessageBox::warning(0,"QFeynCADEditor",message.str().c_str());
    file.close();
    return false;
  }
  file.close();
  QDomNode node=doc.firstChild();
  QDomElement root=node.toElement();
  if (root.isNull()||root.tagName()!=QString("FeynCADrc")) {
    ostringstream message;
    message << "Selected configuration file\n\"" << configFileName
	    << "\"\nis not valid FeynCADrc XML.";
    QMessageBox::warning(0,"QFeynCADEditor",message.str().c_str());
    return false;
  }
  if (root.attribute("version")!=QString("1")) {
    ostringstream message;
    message << "Wrong FeynCADrc XML version in selected input file\n\""
	    << configFileName << "\":\ngot \"" << root.attribute("version")
	    << "\", expected \"1\".";
    QMessageBox::warning(0,"QFeynCADEditor",message.str().c_str());
    return false;
  }
  node=node.firstChild();
  QDomElement preferences=node.toElement();
  bool documentIsOk=true;
  // Solito viscido trucco
  while (1) {
    if (preferences.isNull()||preferences.tagName()!=QString("preferences"))
      { documentIsOk=false; break; }
    node=node.firstChild();
    while (!node.isNull()) {
      QDomElement e=node.toElement();
      if (!e.isNull())
	if (e.tagName()==QString("gridSize")) {
	  QFeynCADEditor::_gridX=
	    e.attribute("x",QString::number(FEYNCAD_DEFAULT_GRID_X)).toUInt();
	  QFeynCADEditor::_gridY=
	    e.attribute("y",QString::number(FEYNCAD_DEFAULT_GRID_Y)).toUInt();
	  if (!QFeynCADEditor::_gridX||!QFeynCADEditor::_gridY) {
	    ostringstream message;
	    message << "In selected configuration file\n\"" << configFileName
		    << "\":\nwrong dimensions for default grid.";
	    QMessageBox::warning(0,"QFeynCADEditor",message.str().c_str());
	    documentIsOk=false;
	    break;
	  }
	} else if (e.tagName()==QString("editor")) {
	  QFeynCADEditor::_gridStep=
	    e.attribute("gridStep",QString::number(FEYNCAD_DEFAULT_GRID_STEP)).
	    toUInt();
	  QFeynCADEditor::_squareBaseSize=
	    e.attribute("squareBaseSize",
			QString::number(FEYNCAD_DEFAULT_SQUARE_BASE_SIZE)).
	    toUInt();
	  QFeynCADEditor::_circleBaseSize=
	    e.attribute("circleBaseSize",
			QString::number(FEYNCAD_DEFAULT_CIRCLE_BASE_SIZE)).
	    toUInt();
	  QFeynCADEditor::_markerStep=
	    e.attribute("markerSize",
			QString::number(FEYNCAD_DEFAULT_MARKER_STEP)).toUInt();
	} else if (e.tagName()==QString("paths")) {
	  QFeynCADEditor::_macrosPath=
	    e.attribute("macros",FEYNCAD_DEFAULT_MACROS_PATH).ascii();
	  QFeynCADEditor::_addendumPath=
	    e.attribute("addendum",FEYNCAD_DEFAULT_ADDENDUM_PATH).ascii();
	  QFeynCADEditor::_metapostPath=
	    e.attribute("metapost",FEYNCAD_DEFAULT_METAPOST_PATH).ascii();
	  QFeynCADEditor::_latexPath=
	    e.attribute("latex",FEYNCAD_DEFAULT_LATEX_PATH).ascii();
	  QFeynCADEditor::_dvipsPath=
	    e.attribute("dvips",FEYNCAD_DEFAULT_DVIPS_PATH).ascii();
	  QFeynCADEditor::_gsPath=
	    e.attribute("gs",FEYNCAD_DEFAULT_GS_PATH).ascii();
	  QFeynCADEditor::_browserPath=
	    e.attribute("browser",FEYNCAD_DEFAULT_BROWSER_PATH).ascii();
	  QFeynCADEditor::_tmpPath=
	    e.attribute("tmp",FEYNCAD_DEFAULT_TMP_PATH).ascii();
	} else if (e.tagName()==QString("miscellaneous")) {
	  QFeynCADEditor::_timeout=
	    e.attribute("timeout",QString::number(FEYNCAD_DEFAULT_TIMEOUT)).
	    toUInt();
	  QFeynCADEditor::_keepFiles=
	    (bool)e.attribute("keepIntermediateFiles",
			      QString::number(FEYNCAD_DEFAULT_KEEP_FILES)).
	    toUInt();
	  QFeynCADEditor::_historyDepth=
	    e.attribute("historyDepth",
			QString::number(FEYNCAD_DEFAULT_HISTORY_DEPTH)).
	    toUInt();
	  QFeynCADEditor::_metapostSettings=
	    e.attribute("metapostSettings",FEYNCAD_DEFAULT_METAPOST_SETTINGS).
	    ascii();
	} else
	  { documentIsOk=false; break; }
      node=node.nextSibling();
    }
    // Necessario
    break;
  }
  if (!documentIsOk) {
    QFeynCADEditor::resetToDefaults();
    ostringstream message;
    message << "Parse error in selected configuration file\n\""
	    << configFileName << "\".\n"
	    << "Preferences have been reset to their default values.";
    QMessageBox::warning(0,"QFeynCADEditor",message.str().c_str());
    return false;
  }
  return true;
}
#undef PROTO_LINE_OR_TADPOLE

// Default statici
int QFeynCADEditor::_gridX=FEYNCAD_DEFAULT_GRID_X;
int QFeynCADEditor::_gridY=FEYNCAD_DEFAULT_GRID_Y;
int QFeynCADEditor::_gridStep=FEYNCAD_DEFAULT_GRID_STEP;
int QFeynCADEditor::_squareBaseSize=FEYNCAD_DEFAULT_SQUARE_BASE_SIZE;
int QFeynCADEditor::_circleBaseSize=FEYNCAD_DEFAULT_CIRCLE_BASE_SIZE;
int QFeynCADEditor::_markerStep=FEYNCAD_DEFAULT_MARKER_STEP;
string QFeynCADEditor::_macrosPath=FEYNCAD_DEFAULT_MACROS_PATH;
string QFeynCADEditor::_addendumPath=FEYNCAD_DEFAULT_ADDENDUM_PATH;
string QFeynCADEditor::_metapostPath=FEYNCAD_DEFAULT_METAPOST_PATH;
string QFeynCADEditor::_latexPath=FEYNCAD_DEFAULT_LATEX_PATH;
string QFeynCADEditor::_dvipsPath=FEYNCAD_DEFAULT_DVIPS_PATH;
string QFeynCADEditor::_gsPath=FEYNCAD_DEFAULT_GS_PATH;
string QFeynCADEditor::_browserPath=FEYNCAD_DEFAULT_BROWSER_PATH;
string QFeynCADEditor::_tmpPath=FEYNCAD_DEFAULT_TMP_PATH;
int QFeynCADEditor::_timeout=FEYNCAD_DEFAULT_TIMEOUT;
bool QFeynCADEditor::_keepFiles=FEYNCAD_DEFAULT_KEEP_FILES;
size_t QFeynCADEditor::_historyDepth=FEYNCAD_DEFAULT_HISTORY_DEPTH;
string QFeynCADEditor::_metapostSettings=FEYNCAD_DEFAULT_METAPOST_SETTINGS;
//  Default per gli oggetti
vertex QFeynCADEditor::_vertex(make_pair(-1,-1));
line QFeynCADEditor::_line(make_pair(0,-1),make_pair(-1,0));
line QFeynCADEditor::_tadpole(make_pair(-1,-1));
tag QFeynCADEditor::_tag(make_pair(-1,-1));
//  La clipboard
status QFeynCADEditor::_clipboard;

#undef FEYNCAD_DEFAULT_GRID_X
#undef FEYNCAD_DEFAULT_GRID_Y
#undef FEYNCAD_DEFAULT_GRID_STEP
#undef FEYNCAD_DEFAULT_SQUARE_BASE_SIZE
#undef FEYNCAD_DEFAULT_CIRCLE_BASE_SIZE
#undef FEYNCAD_DEFAULT_MARKER_STEP
#undef FEYNCAD_DEFAULT_FONT_NAME
#undef FEYNCAD_DEFAULT_MACROS_PATH
#undef FEYNCAD_DEFAULT_ADDENDUM_PATH
#undef FEYNCAD_DEFAULT_METAPOST_PATH
#undef FEYNCAD_DEFAULT_LATEX_PATH
#undef FEYNCAD_DEFAULT_DVIPS_PATH
#undef FEYNCAD_DEFAULT_GS_PATH
#undef FEYNCAD_DEFAULT_BROWSER_PATH
#undef FEYNCAD_DEFAULT_TMP_PATH
#undef FEYNCAD_DEFAULT_TIMEOUT
#undef FEYNCAD_DEFAULT_KEEP_FILES
#undef FEYNCAD_DEFAULT_HISTORY_DEPTH
#undef FEYNCAD_DEFAULT_METAPOST_SETTINGS
