// QFeynCADEditor.h
// Copyright (C) 1999-2004 Paolo Ribeca

// This file is part of FeynCAD package.

// FeynCAD is free software;
// you can redistribute it and/or modify it under the terms
// of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with FeynCAD; see the file COPYING.
// If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <set>
#include <utility>
#include <vector>
#include <deque>

#include <qwidget.h>
#include <qevent.h>
#include <qpopupmenu.h>
#include <qtimer.h>
#include <qaccel.h>

using namespace std;

#include "QFeynCADVertexEdit.h"
#include "QFeynCADTabDialog.h"
#include "QFeynCADLineEdit.h"
#include "QFeynCADTadpoleEdit.h"
#include "QFeynCADTagEdit.h"

#ifndef FEYNCAD_EDITOR_GUARD
#define FEYNCAD_EDITOR_GUARD

#include "structures.C"

struct status {
 public:
  pair<int,int> gridSize;
  //
  set<vertex> vertices;
  multiset<line> lines;
  set<tag> tags;
 public:
  status(): gridSize(make_pair(0,0)) { }
  void clear() {
    this->gridSize=make_pair(0,0);
    this->vertices.clear();
    this->lines.clear();
    this->tags.clear();
  }
  size_t size()
  { return this->vertices.size()+this->lines.size()+this->tags.size(); }
};

class QFeynCADTransformEdit; // Bleah

#include "QFeynCADTransformEdit.h"

class QFeynCADEditor:
  public QWidget
{
  Q_OBJECT
 public:
  typedef enum { DrawMode, SelectMode, PasteMode } mode;
  typedef enum { Nothing, Vertex, Line, Tag } last;
 protected:
  static int _gridX,_gridY;
  static int _gridStep;
  static int _squareBaseSize;
  static int _circleBaseSize;
  static int _markerStep;
  //
  status _status,_alpha;
  mode _mode;
  pair<int,int> _memoryOne,_memoryTwo,_lastOne,_lastTwo;
  last _last;
  bool _tracingLine;
  //
  pair<int,int> _selRectOne,_selRectTwo;
  pair<int,int> _newSelRectOne,_newSelRectTwo;
  pair<int,int> _newSelRectRotationCenter;
  double _newSelRectRotationAngle;
  //
  deque<pair<status,string> > _history;
  size_t _current;
  static size_t _historyDepth;
  //
  QAccel* _accelerator;
  //
  QPopupMenu* _modifyOrDelete;
  QLabel* _modifyOrDeleteTitle;
  QFeynCADVertexEdit* _modifyVertex;
  QFeynCADTabDialog* _modifyLineParent;
  vector<pair<QFeynCADLineEdit*,QFeynCADTadpoleEdit*> > _modifyLine;
  QFeynCADTagEdit* _modifyTag;
  QFeynCADTransformEdit* _modifySelection;
  QTimer* _runawayDialogCatcher;
  //
  QString _creationDateTime,_modificationDateTime;
  QString _author,_comments;
  // E' il nome del file temporaneo
  string _fileName;
  //
  static string _fontName;
  static string _macrosPath;
  static string _addendumPath;
  static string _metapostPath;
  static string _latexPath;
  static string _dvipsPath;
  static string _gsPath;
  static string _browserPath;
  static string _tmpPath;
  static int _timeout;
  static bool _keepFiles;
  static string _metapostSettings;
  //
  static vertex _vertex;
  static line _line,_tadpole;
  static tag _tag;
  //
  static status _clipboard;
  //
  bool _isUpToDate;
  // Controls the emission of GUI messages in some member functions
  bool _inBatchMode;
 protected:
  inline int xGridPosWhenXCanvasIs(int xCanvasPos) const;
  inline int yGridPosWhenYCanvasIs(int yCanvasPos) const;
  inline int xCanvasPosWhenXGridIs(int xGridPos) const;
  inline int yCanvasPosWhenYGridIs(int yGridPos) const;
  inline QRect
    getEnvelope(const pair<int,int>& one,const pair<int,int>& two) const;
  //
  void paintEvent(QPaintEvent *e);
  void mouseMoveEvent(QMouseEvent * e);
  void mousePressEvent(QMouseEvent * e);
  void mouseReleaseEvent(QMouseEvent * e);
  void translateAlpha(int x,int y);
 protected slots:
  void pushHistory(string description);
  void updateVertexList();
  void updateLineList();
  void updateTagList();
  void updateModifyOrDelete();
  void deleteVertex();
  void addLine();
  void deleteLines();
  void deleteTag();
  void prepareModifyVertex();
  void prepareModifyLines();
  void prepareModifyTag();
  void modifyVertex();
  void modifyLines();
  void modifyTag();
  void editLast();
  void acceptPaste();
  void abandonPaste();
  void clipToSelection();
  void rotateAlpha(int,int,int);
  void acceptRotation();
  void abandonTransformation();
  bool mergeAlphaUniquifying();
  void stopRunawayDialogCatcher()
  { this->_runawayDialogCatcher->stop(); emit message(false,""); return; }
  void reenableApp();
  bool updatePostscript();
 public:
  QFeynCADEditor(QWidget* parent=0,const char* name=0,
		 int gridSizeX=_gridX,int gridSizeY=_gridY,string fileName="");
  ~QFeynCADEditor();
  //
  void setBatchMode() { this->_inBatchMode=true; return; }
  static string gsPath() { return QFeynCADEditor::_gsPath; }
  static string tmpPath() { return QFeynCADEditor::_tmpPath; }
  static unsigned timeout() { return QFeynCADEditor::_timeout; }
 signals:
  void gridChanged(unsigned,unsigned);
  void newGridPosition(int,int);
  void viewerUpdateReq(unsigned,unsigned);
  void undoRedoUpdateReq(string undo,string redo);
  void vertexListUpdateReq(const QStringList& vertexList);
  void lineListUpdateReq(const QStringList& lineList);
  void tagListUpdateReq(const QStringList& tagList);
  void setMainAppEnabledReq(bool);
  void titleUpdateReq();
  void newMode();
  void newSelection();
  void message(bool isPermanent,string m);
 public slots:
  bool setFileNameTo(string fileName)
  { this->_fileName=fileName; return this->updatePostscript(); }
  void undoRedoUpdate();
  void goBackInHistory();
  void goForwardInHistory();
  void setModeTo(QFeynCADEditor::mode m);
  void deleteSelection();
  void dumpTo(string fcdFileName);
  bool readFromFile(string fcdFileName);
  void readFromBuffer(string fcdBufferName);
  void exportEpsTo(string tmpFileName,string epsFileName);
  void selectVertexWhenListIs(int comboProgr);
  void selectLineWhenListIs(int comboProgr);
  void selectTagWhenListIs(int comboProgr);
  //
  void setGridSizeTo(int x,int y);
  // Configuration-related issues
  static void resetToDefaults();
  static void dumpConfigTo(string configFileName);
  static bool readConfigFrom(string configFileName);
  //
  friend class page;
  friend class QFeynCADCombined;
  friend class QFeynCADMain;
  friend class QFeynCADConfigEdit;
  friend class QFeynCADTransformEdit;
  friend class QFeynCADInfos;
  friend class QFeynCADHelp;
};

#endif
