// QPsWidget.C
// Copyright (C) 1999-2004 Paolo Ribeca

// This file is part of FeynCAD package.

// FeynCAD is free software;
// you can redistribute it and/or modify it under the terms
// of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with FeynCAD; see the file COPYING.
// If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

// Implementation of ghostscript-related handshaking is borrowed from
//  some 3.x.x versions of (kghostview).

#define Q_PS_WIDGET_DEFAULT_GS_PATH "gs"
#define Q_PS_WIDGET_DEFAULT_TIMEOUT 5

//#define Q_PS_WIDGET_DEBUG

// Quick 'n dirty implementation di un widget che visualizza un file (eps).
//  Il widget e' dotato di un solo controllo [l'ingrandimento] e di un file
//  selector [che pero' e' nascosto nel caso in cui il widget venga creato
//  in modalita' embedded].

/*

Ghostview interface to ghostscript

When the GHOSTVIEW environment variable is set, ghostscript draws on
an existing drawable rather than creating its own window.  Ghostscript
can be directed to draw on either a window or a pixmap.

Drawing on a Window

The GHOSTVIEW environment variable contains the window id of the target
window.  The window id is an integer.  Ghostscript will use the attributes
of the window to obtain the width, height, colormap, screen, and visual of
the window. The remainder of the information is gotten from the GHOSTVIEW
property on that window.


Drawing on a Pixmap

The GHOSTVIEW environment variable contains a window id and a pixmap id.
They are integers separated by white space.  Ghostscript will use the
attributes of the window to obtain the colormap, screen, and visual to use.
The width and height will be obtained from the pixmap. The remainder of the
information, is gotten from the GHOSTVIEW property on the window.  In this
case, the property is deleted when read.

The GHOSTVIEW environment variable

parameters:	window-id [pixmap-id]

scanf format:	"%d %d"

explanation of parameters:

	window-id: tells ghostscript where to
		    - read the GHOSTVIEW property
		    - send events
		    If pixmap-id is not present,
		    ghostscript will draw on this window.

	pixmap-id: If present, tells ghostscript that a pixmap will be used
		    as the final destination for drawing.  The window will
		    not be touched for drawing purposes.

The GHOSTVIEW property

type:	STRING

parameters:

    bpixmap orient llx lly urx ury xdpi ydpi [left bottom top right]

scanf format: "%d %d %d %d %d %d %f %f %d %d %d %d"

explanation of parameters:

	bpixmap: pixmap id of the backing pixmap for the window.  If no
		pixmap is to be used, this parameter should be zero.  This
		parameter must be zero when drawing on a pixmap.

	orient:	orientation of the page.  The number represents clockwise
		rotation of the paper in degrees.  Permitted values are
		0, 90, 180, 270.

	llx, lly, urx, ury: Bounding box of the drawable.  The bounding box
		is specified in PostScript points in default user coordinates.
		Wilco: Note the word *drawable*. This means that this bounding
		       box is generally not the same as the BoundingBox
		       specified in the DSC. In case DocumentMedia is
		       specified, it is equal to the Media's bounding box.

	xdpi, ydpi: Resolution of window.  (This can be derived from the
		other parameters, but not without roundoff error.  These
		values are included to avoid this error.)

	left, bottom, top, right: (optional)
		Margins around the window.  The margins extend the imageable
		area beyond the boundaries of the window.  This is primarily
		used for popup zoom windows.  I have encountered several
		instances of PostScript programs that position themselves
		with respect to the imageable area.  The margins are specified
		in PostScript points.  If omitted, the margins are assumed to
		be 0.

Events from ghostscript

If the final destination is a pixmap, the client will get a property notify
event when ghostscript reads the GHOSTVIEW property causing it to be deleted.

Ghostscript sends events to the window where it read the GHOSTVIEW property.
These events are of type ClientMessage.  The message_type is set to
either PAGE or DONE.  The first long data value gives the window to be used
to send replies to ghostscript.  The second long data value gives the primary
drawable.  If rendering to a pixmap, it is the primary drawable.  If rendering
to a window, the backing pixmap is the primary drawable.  If no backing pixmap
is employed, then the window is the primary drawable.  This field is necessary
to distinguish multiple ghostscripts rendering to separate pixmaps where the
GHOSTVIEW property was placed on the same window.

The PAGE message indicates that a "page" has completed.  Ghostscript will
wait until it receives a ClientMessage whose message_type is NEXT before
continuing.

The DONE message indicates that ghostscript has finished processing.

*/

#include <string>
#include <sstream>
#include <iostream>
#include <fstream>

#include <qapplication.h>
#include <qwidget.h>
#include <qprocess.h>
#include <qpaintdevicemetrics.h>
#include <qpixmap.h>
#include <qstring.h>
#include <qmessagebox.h>

#include <X11/Xlib.h>
#include <X11/Xatom.h>

#include "QPsWidget.h"

int handler(Display* d,XErrorEvent* e) {
  char msg[80],req[80],number[80];
  XGetErrorText(d,e->error_code,msg,sizeof(msg));
  sprintf(number,"%d",e->request_code);
  XGetErrorDatabaseText(d,"XRequest",number,"<unknown>",req,sizeof(req));
  return 0;
}

QStringList readUnixEnvironment() {
  QStringList res;
  if (system("env > ./.environment.tmp")==-1) {
    QMessageBox::warning(0,"readUnixEnvironment",
			 "Could not read Unix environment.");
    return res;
  }
  ifstream env("./.environment.tmp");
  string assignment;
  while (env) {
    env >> assignment;
    res+=QString(assignment.c_str());
  }
  if (system("rm -f ./.environment.tmp")==-1) {
    QMessageBox::warning(0,"readUnixEnvironment",
			 "Could not delete temporary file "
			 "\"./.environment.tmp\".");
    return res;
  }
  return res;
}

QPsWidget::QPsWidget(QWidget* parent,const char* name,
		     unsigned requestedWidth,unsigned requestedHeight,
		     string fileName):
  QWidget(parent,name),
  _backgroundPixmap(0),
  _requestedWidth(requestedWidth),
  _requestedHeight(requestedHeight),
  _fileName(fileName),
  _process(0),
  _isBusy(false),
  _isReqPending(false)
{
  // X-related stuff
  XSetErrorHandler(handler);
  const char* atomNames[]=
    { "GHOSTVIEW","GHOSTVIEW_COLORS","NEXT","PAGE","DONE" };
  XInternAtoms(x11Display(),const_cast<char**>(atomNames),5,false,
	       this->_atoms);
  // Get the screen resolution.
  QPaintDeviceMetrics qpdm(this);
  this->_logicalDpi=make_pair(qpdm.logicalDpiX(),qpdm.logicalDpiY());
  //
  connect(&this->_timeoutTimer,SIGNAL(timeout()),this,SLOT(timeout()));
  //
  this->setFixedSize(this->_requestedWidth,this->_requestedHeight);
  this->redrawPs();
}

QPsWidget::~QPsWidget()
{ this->stopInterpreter(); }

void QPsWidget::outputErrors() {
  QString message;
  message+=this->_process->readStdout();
  message+=this->_process->readStderr();
  QMessageBox::warning(this,"QPsWidget",message);
}

void QPsWidget::setFileTo(string fileName) {
#ifdef Q_PS_WIDGET_DEBUG
  cout << "NEW FILE NAME: " << this->_requestedWidth << ","
       << this->_requestedHeight << endl;
#endif
  // In principle one could do some checks here...
  //
  if (fileName!=this->_fileName)
    this->_fileName=fileName;
  return;
}

bool QPsWidget::startInterpreter() {
  if (this->_process) {
    QMessageBox::warning(this,"QPsWidget",
			 "Ghostscript subprocess was not stopped before.");
    return false;
  }
  //
  ostringstream id,command;
  id << "GHOSTVIEW=" << winId();
  command << QPsWidget::_ghostscriptPath
	  << " -dQUIET -dNOPAUSE -dSAFER -sDEVICE=x11alpha "
	  << this->_fileName << " -c quit";
  //
  qApp->flushX();
  //
  QStringList args=QStringList::split(" ",QString(command.str().c_str())),
    env=readUnixEnvironment();
  env+=QString(id.str().c_str());
  //
  this->_process=new QProcess(args);
  connect(this->_process,SIGNAL(readyReadStdout()),this,SLOT(outputErrors()));
  connect(this->_process,SIGNAL(readyReadStderr()),this,SLOT(outputErrors()));
  //
  if (!this->_process->start(&env))
    /*
    QMessageBox::warning(this,"QPsWidget",
			 "Could not start Ghostscript subprocess.");
    */
    return false;
  //
  //cout << "PID=" << this->_process->processIdentifier() << endl;
  //this->_isBusy=true;
  setCursor(waitCursor);
  return true;
}

bool QPsWidget::stopInterpreter() {
  if (!this->_process)
    return true;
  // We do not need to display any following pages
  /*
  XEvent e;
  e.xclient.type = ClientMessage;
  e.xclient.display = x11Display();
  e.xclient.window = _gsWindow;
  e.xclient.message_type = _atoms[NEXT];
  e.xclient.format = 32;
  //
  XSendEvent( x11Display(), _gsWindow, false, 0, &e );
  XFlush( x11Display() );
  */
  this->_process->kill();
  QTime time;
  time.start();
  while (unsigned(time.elapsed())<(1000*QPsWidget::_timeout)&&
	 this->_process->isRunning())
    qApp->processEvents();
  if (this->_process->isRunning())
    /*
    QMessageBox::warning(this,"QPsWidget",
			 "Could not stop Ghostscript subprocess.");
    */
    return false;
  delete this->_process;
  this->_process=0;
  // (_gsWindow) e' dove finiscono i messaggi di Ghostscript
  this->_gsWindow=None;
  //
  return true;
}

// Per evitare cicli infiniti, in caso di anomalie lo stato viene
//  cancellato completamente; in particolare, le eventuali richieste
//  di respawn vengono resettate
#define Q_PS_WIDGET_CLEAR_STATUS \
  setCursor(arrowCursor);\
  this->_isBusy=false;\
  this->_isReqPending=false;
//

// Qui la cosa dovrebbe funzionare che in caso di anomalia,
//  e di dimensioni specificate, il pixmap si ridisegna bianco
bool QPsWidget::redrawPs(unsigned requestedWidth,unsigned requestedHeight) {
  // Si testa se le dimensioni sono cambiate (eventuali modifiche
  //  devono essere registrate anche in caso di deferred execution)
  if (requestedWidth&&requestedHeight&&
      (requestedWidth!=this->_requestedWidth||
       requestedHeight!=this->_requestedHeight)) {
    this->_requestedWidth=requestedWidth;
    this->_requestedHeight=requestedHeight;
  }
  // Ridipinge il widget sulla base dei valori correnti di file,
  //  ingrandimento e dimensioni richieste
  if (this->_isBusy)
    { this->_isReqPending=true; return false; }
  else
    this->_isBusy=true;
  //
  if (!this->stopInterpreter()) {
    QMessageBox::warning(this,"QPsWidget","Could not stop Postscript "
			 "interpreter.");
#ifdef Q_PS_WIDGET_DEBUG
    cout << "SIGNAL: PAGE ERROR" << endl;
#endif
    emit pageError();
    Q_PS_WIDGET_CLEAR_STATUS
    return false;
  }
  //
  if (this->_fileName=="") {
#ifdef Q_PS_WIDGET_DEBUG
    cout << "BLANK" << endl;
#endif
    // Per l'invariante :)
    this->_xMagnification=1.,this->_yMagnification=1.;
    // Si limita a dipingere il pixmap bianco
    qApp->processEvents();
    this->setPaletteBackgroundColor(white);
    this->erase();
    this->setFixedSize(this->_requestedWidth,this->_requestedHeight);
    this->_isBusy=false;
    if (this->_isReqPending)
      { this->_isReqPending=false; this->redrawPs(); }
    return true;
  }
  //
  ifstream eps(this->_fileName.c_str());
  if (!eps) {
    ostringstream message;
    message << "Could not open requested EPS file\n\"" << this->_fileName
	    << "\".";
    QMessageBox::warning(this,"QPsWidget",message.str().c_str());
#ifdef Q_PS_WIDGET_DEBUG
    cout << "SIGNAL: PAGE ERROR" << endl;
#endif
    emit pageError();
    Q_PS_WIDGET_CLEAR_STATUS
    return false;
  }
  // Si legge il bounding box
  char dum;
  string toBeFound="%%BoundingBox:";
  while (eps) {
    // Ci si posiziona all'inizio
    do { eps >> dum; }
    while (dum!=toBeFound.c_str()[0]);
    // Si prova ad avanzare
    register size_t i;
    for (i=1;i<toBeFound.size();++i) {
      eps >> dum;
      if (dum!=toBeFound.c_str()[i]) {
	for (register size_t j=0;j<i;++j)
	  eps.unget();
	break;
      }
    }
    if (i==toBeFound.size())
      break;
  }
  if (!eps) {
    QMessageBox::warning(this,"QPsWidget","Could not read bounding box "
			 "dimensions for requested EPS file.");
#ifdef Q_PS_WIDGET_DEBUG
    cout << "SIGNAL: PAGE ERROR" << endl;
#endif
    emit pageError();
    Q_PS_WIDGET_CLEAR_STATUS
    return false;
  }
  //
  int a,b,c,d;
  eps >> a >> b >> c >> d;
  this->_boundingBox.setTo(a,b,c,d);
  eps.close();
  //
  this->_xMagnification=
    (this->_requestedWidth?
     float(this->_requestedWidth)/this->_boundingBox.width()*72./
     this->_logicalDpi.first:1.);
  this->_yMagnification=
    (this->_requestedHeight?
     float(this->_requestedHeight)/this->_boundingBox.height()*72./
     this->_logicalDpi.second:1.);
  int newWidth=int(this->_xMagnification*this->_boundingBox.width()/72.*
		   this->_logicalDpi.first+0.5);
  int newHeight=int(this->_yMagnification*this->_boundingBox.height()/72.*
		    this->_logicalDpi.second+0.5);
  if (this->_backgroundPixmap)
    delete this->_backgroundPixmap;
  this->_backgroundPixmap=new QPixmap(newWidth,newHeight);
  this->_backgroundPixmap->fill(white);
  this->setErasePixmap(*this->_backgroundPixmap);
  this->erase();
  this->setFixedSize(newWidth,newHeight);
  this->repaint();
  qApp->processEvents();
#ifdef Q_PS_WIDGET_DEBUG
  cout << "REDRAW PS: " << newWidth << " " << newHeight << endl;
#endif
  //
  char buf[512];
  sprintf(buf,"%ld %d %d %d %d %d %g %g",this->_backgroundPixmap->handle(),0,
	  this->_boundingBox.one,this->_boundingBox.two,
	  this->_boundingBox.thr,this->_boundingBox.fou,
	  (float)this->_logicalDpi.first*this->_xMagnification,
	  (float)this->_logicalDpi.second*this->_yMagnification);
  XChangeProperty(x11Display(),winId(),_atoms[GHOSTVIEW],XA_STRING,8,
		  PropModeReplace,(unsigned char*)buf,strlen(buf));
  sprintf(buf,"%s %d %d","Color",
	  (int)BlackPixel(x11Display(),DefaultScreen(x11Display())),
	  (int)WhitePixel(x11Display(),DefaultScreen(x11Display())));
  XChangeProperty(x11Display(),winId(),_atoms[GHOSTVIEW_COLORS],XA_STRING,8,
		  PropModeReplace,(unsigned char*)buf,strlen(buf));
  // Make sure the properties are updated immediately.
  XSync(x11Display(),false);
  //
  this->repaint();
  //
  emit pageStarted();
#ifdef Q_PS_WIDGET_DEBUG
  cout << "SIGNAL: PAGE STARTED" << endl;
#endif
  if (!this->startInterpreter()) {
    QMessageBox::warning(this,"QPsWidget","Could not start Postscript "
			 "interpreter.");
#ifdef Q_PS_WIDGET_DEBUG
    cout << "SIGNAL: PAGE ERROR" << endl;
#endif
    emit pageError();
    Q_PS_WIDGET_CLEAR_STATUS
    return false;
  }
  // Qui il widget e' ancora busy, e lo resta fino al segnale di (gs),
  //  o fino ad un eventuale timeout
  this->_timeoutTimer.start(1000*QPsWidget::_timeout,true);
  return true;
}

void QPsWidget::timeout() {
  this->stopInterpreter();
  // We blank the page
  if (this->_backgroundPixmap)
    delete this->_backgroundPixmap;
  this->_backgroundPixmap=
    new QPixmap(this->_requestedWidth,this->_requestedHeight);
  this->_backgroundPixmap->fill(white);
  this->setErasePixmap(*this->_backgroundPixmap);
  this->erase();
  //
#ifdef Q_PS_WIDGET_DEBUG
  cout << "SIGNAL: PAGE TIMEOUT" << endl;
#endif
  emit pageTimeout();
  Q_PS_WIDGET_CLEAR_STATUS
  return;
}

bool QPsWidget::x11Event(XEvent* e) {
  // I casi di perdita di sincronia sono quelli in cui questi segnali
  //  per qualche motivo non vengono ricevuti
  if( e->type == ClientMessage ) {
    _gsWindow = e->xclient.data.l[0];
    //
    if( e->xclient.message_type == _atoms[PAGE] ) {
      //cout << "QPsWidget: received PAGE" << endl;
      this->_timeoutTimer.stop();
#ifdef Q_PS_WIDGET_DEBUG
      cout << "SIGNAL: PAGE COMPLETED" << endl;
#endif
      emit pageCompleted();
      setCursor(arrowCursor);
      this->_isBusy=false;
      if (this->_isReqPending)
	{ this->_isReqPending=false; this->redrawPs(); }
      return true;
    } else if( e->xclient.message_type == _atoms[DONE] ) {
      //cout << "QPsWidget: received DONE" << endl;
      this->_timeoutTimer.stop();
#ifdef Q_PS_WIDGET_DEBUG
      cout << "SIGNAL: PAGE COMPLETED" << endl;
#endif
      emit pageCompleted();
      setCursor(arrowCursor);
      this->_isBusy=false;
      if (this->_isReqPending)
	{ this->_isReqPending=false; this->redrawPs(); }
      return true;
    }
  }
  return QWidget::x11Event( e );
}

#undef Q_PS_WIDGET_CLEAR_STATUS

string QPsWidget::_ghostscriptPath=Q_PS_WIDGET_DEFAULT_GS_PATH;
unsigned QPsWidget::_timeout=Q_PS_WIDGET_DEFAULT_TIMEOUT;

#undef Q_PS_WIDGET_DEFAULT_GS_PATH
#undef Q_PS_WIDGET_DEFAULT_TIMEOUT

#ifdef Q_PS_WIDGET_DEBUG
#undef Q_PS_WIDGET_DEBUG
#endif
