// main.C
// Copyright (C) 1999-2004 Paolo Ribeca

// This file is part of FeynCAD package.

// FeynCAD is free software;
// you can redistribute it and/or modify it under the terms
// of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with FeynCAD; see the file COPYING.
// If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <unistd.h>
#include <getopt.h>
#include <qapplication.h>
#include <qsplashscreen.h>
#include <qstring.h>
#include <qdir.h>
#include <qmessagebox.h>
#include "QFeynCADMain.h"

using namespace std;

// In this function we check that there are no overflows in a value
//   which has been introduced by the user in a syntactically correct way.
//  The acceptable range is specified during the call;
//   eventually, the error message is returned
pair<QString,string>
revalidate(const QString& defText,const double& def,
	   const double& min,const double& max,
	   const string& valueName,const QString& value) {
  // La stringa contiene il testo di default: la si lascia inalterata
  if (defText!=QString::null&&value==defText)
    return make_pair(value,"");
  double dValue=value.toDouble();
  // La stringa contiene il _valore_ di default:
  //  la si riporta al _testo_ di default
  if (defText!=QString::null&&dValue==def)
    return make_pair(defText,"");
  if (dValue>=min&&dValue<=max)
    return make_pair(value,"");
  // Qui cominciano ad esserci dei problemi
  ostringstream message;
  message << "Value of \"" << valueName << "\" was out of range (" << dValue
	  << ", should be between " << min << " and " << max << ")." << endl;
  if (dValue<min) {
    message << "  ==>Quantity \"" << valueName << "\" has been raised to "
	    << min << ".<==" << endl;
    return make_pair(min==def?defText:QString::number(min),message.str());
  } else {
    message << "  ==>Quantity \"" << valueName
	    << "\" has been lowered to " << max << ".<==" << endl;
    return make_pair(max==def?defText:QString::number(max),message.str());
  }
}
// In this function we double commata, to produce strings
//  compatible to feynmp macros
string taint(const string& metaPostString) {
  QString orig(metaPostString.c_str());
  return orig.replace(",",",,").ascii();
}

#include "images/splashscreen.xpm"

unsigned page::_globalCounter=1;

// Le opzioni sono come segue:
//  -b --batch per ./FeynCAD file.fcd file.eps, no interfaccia grafica
//  -c --configFile= per specificare preference file diverso da ~/.FeynCADrc
//  -h --help
//  Se -b non e' specificata, tutti gli altri argomenti vengono aperti

int main(int argc,char **argv) {
  QApplication a(argc,argv);
  QSplashScreen *splash=new QSplashScreen(QPixmap(splashscreen_xpm));
  //
  string defaultPreferencePath=(QDir::homeDirPath()+"/.FeynCADrc").ascii(),
    preferencePath=defaultPreferencePath;
  // Si processano le opzioni sulla riga di comando
  string banner=
    "\n FeynCAD v1pre1 - (c) 1999-2004 Paolo Ribeca,\n"
    "   <software@paoloribeca.net>.\n\n";
  string usage=
    " Usage:\n\n"
    "  FeynCAD [Qt-options] [FeynCAD-options] [file1 file2 ...]\n"
    "      Invokes FeynCAD in graphical mode.\n"
    "      All specified files are open.\n\n"
    "  FeynCAD -b [Qt-options] [FeynCAD-options] inputFile outputFile\n"
    "      Invokes FeynCAD in batch mode.\n"
    "      Transforms FeynCAD-XML file \"inputFile\" "
    "into EPS file \"outputFile\".\n"
    "      The suffix \".eps\" is appended to \"outputFile\" "
    "if not present.\n\n"
    "  [FeynCAD-options] can be one or more of\n"
    "    -c configFile\n"
    "    --config-file configFile\n"
    "      Specifies an alternative configuration file "
    "(default is \"~/.FeynCADrc\").\n"
    "      File \"configFile\" must be a valid FeynCADrc-XML file.\n"
    "    -h\n"
    "    --help\n"
    "      Displays these informations and exits.\n\n";
  static option longOptions[] = {
    {"config-file",1,0,'c'},
    {"batch",0,0,'b'},
    {"help",0,0,'h'},
    {0,0,0,0}
  };
  extern int optind;
  bool batchMode=false;
  int optionIndex=0;
  register int par;
  while ((par=getopt_long_only(argc,argv,"c:bh",longOptions,&optionIndex))!=-1)
    switch (par) {
    case 0:
      // Long options pure
      break;
      // Short options
    case 'c': preferencePath=optarg; break;
    case 'b': batchMode=true; break;
    case 'h': cout << banner << usage; return 0; break;
    case '?':
      cout << banner << " Error: Wrong syntax of parameters." << endl << endl
	   << usage;
      return 1;
      break;
    }
  // Si prova a leggere il file che contiene le preferenze
  if (!QFeynCADEditor::readConfigFrom(preferencePath))
    if (defaultPreferencePath!=preferencePath) {
      if (!QFeynCADEditor::readConfigFrom(defaultPreferencePath))
	QFeynCADEditor::resetToDefaults();
    } else
      QFeynCADEditor::resetToDefaults();
  QPsWidget::setGhostscriptPathTo(QFeynCADEditor::gsPath());
  QPsWidget::setTimeoutTo(QFeynCADEditor::timeout());
  if (batchMode) {
    // In questo caso niente interfaccia grafica
    QFeynCADEditor editor;
    editor.setBatchMode();
    if (argc-optind!=2) {
      cout << banner
	   << " Error: Wrong number of parameters in batch mode (2 needed)."
	   << endl << endl << usage;
      return 2;
    }
    string completeInputFileName=QFileInfo(argv[optind]).absFilePath().ascii();
    // Il nome del file EPS deve essere pretrattato (puo' non avere suffisso)
    string epsFileName(argv[optind+1]);
    if (epsFileName.size()>4&&
	epsFileName[epsFileName.size()-4]=='.'&&
	epsFileName[epsFileName.size()-3]=='e'&&
	epsFileName[epsFileName.size()-2]=='p'&&
	epsFileName[epsFileName.size()-1]=='s')
      argv[optind+1][epsFileName.size()-4]='\0';
    //  A questo punto in (argv[optind+1]) c'e' il nome troncato    
    string outputFileNameWithSuffix=string(argv[optind+1])+".eps";
    string completeOutputFileName=
      QFileInfo(outputFileNameWithSuffix.c_str()).absFilePath().ascii();
    //
    cout << banner << " Processing one FeynCAD file in batch mode:" << endl;
    cout << "  Reading specified file \"" << argv[optind] << "\"..." << flush;
    if (editor.readFromFile(completeInputFileName))
      cout << " done." << endl << flush;
    else {
      cout << " no such file, or some error in it!" << endl
	   << " Bye, sorry." << endl << endl;
      return 3;
    }
    cout << "  Writing EPS to specified file \"" << outputFileNameWithSuffix
	 << "\"..." << flush;
    // Qui bisogna cambiare directory, alla temporanea
    if (chdir(QFeynCADEditor::tmpPath().c_str())) {
      cout << " could not change directory to \"" << QFeynCADEditor::tmpPath()
	   << "\"!" << endl << " Bye, sorry." << endl << endl;
      return 4;
    }
    // Il file che associamo all'editor deve essere un temporaneo,
    //  altrimenti il risultato verra' cancellato all'uscita
    ostringstream tmpFileName;
    tmpFileName << QFeynCADEditor::tmpPath() << "/FeynCAD-" << getpid()
		<< "-0";
    if (!editor.setFileNameTo(tmpFileName.str())) {
      cout << " some error while processing!" << endl
	   << " Bye, sorry." << endl << endl;
      return 5;
    }
    ostringstream command;
    command << "mv " << tmpFileName.str() << ".eps " << completeOutputFileName
	    << " >& /dev/null 2>&1";
    if (system(command.str().c_str())) {
      cout << " could not open file for writing!" << endl
	   << " Bye, sorry." << endl << endl;
      return 6;
    }
    cout << " done." << endl << flush << " Bye." << endl << endl;
    return 0;
  }
  // Screensplash
  splash->show();
  // Si crea un catalogo dei file specificati sulla riga di comando
  multiset<string> files;
  QFileInfo fileInfos;
  for (register int i=optind;i<argc;++i) {
    fileInfos.setFile(argv[i]);
    if (fileInfos.exists())
      files.insert(fileInfos.absFilePath().ascii());
    else {
      ostringstream message;
      message << "Specified file\n\"" << argv[i]
	      << "\"\ndoes not exist.";
      QMessageBox::warning(0,"FeynCAD",message.str().c_str());
    }
  }
  // Attenzione: come effetto di bordo si cambia directory, alla (tmp)
  QFeynCADMain w;
  a.setMainWidget(&w);
  // Si aprono i file
  register unsigned goodFiles=0;
  for (set<string>::const_iterator i=files.begin();i!=files.end();++i)
    if (w.doFileOpen(*i))
      ++goodFiles;
  if (!goodFiles)
    w.fileNew();
  w.displayMessage(true,"Utilization of this program is subject to GNU GPL "
		   "license. Open menu \"Help->About FeynCAD->License\" "
		   "if you need further details.");
  QTimer::singleShot(3000,splash,SLOT(close()));
  QTimer::singleShot(3000,&w,SLOT(show()));
  //
  a.exec();
  // Si salva il file che contiene le preferenze
  QFeynCADEditor::dumpConfigTo(preferencePath);
  return 0;
}
