// structures.C
// Copyright (C) 1999-2004 Paolo Ribeca

// This file is part of FeynCAD package.

// FeynCAD is free software;
// you can redistribute it and/or modify it under the terms
// of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with FeynCAD; see the file COPYING.
// If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FEYNCAD_STRUCTURES_GUARD
#define FEYNCAD_STRUCTURES_GUARD

#include <string>
#include <utility>
#include <iostream>
#include <sstream>

#include <qdom.h>
#include <qstring.h>

#define UNIT "mm" //"2.84528pt"
// 4095.9999
#define MPOST_CEIL 3000
// 4095.9999/0.99626/1.5 \approx 2741
#define MPOST_THIN_CEIL 2000
// 4095.9999/0.99626/2.83464 \approx 1450
#define MPOST_MM_CEIL 1000

#define FEYNCAD_ANGLE_REVALIDATOR "",0,-360,360
#define FEYNCAD_LINE_ANGLE_REVALIDATOR "",-361,-360,360
#define FEYNCAD_THIN_REVALIDATOR QString::null,0,0,MPOST_THIN_CEIL
#define FEYNCAD_TENSION_REVALIDATOR "",1,0.75,MPOST_CEIL
#define FEYNCAD_EXTENSION_REVALIDATOR "",4,0,MPOST_MM_CEIL
#define FEYNCAD_RUBOUT_REVALIDATOR "",1,1,MPOST_CEIL
#define FEYNCAD_SCALE_REVALIDATOR QString::null,0,0,MPOST_CEIL

using namespace std;

// In this function we check that there are no overflows in a value
//   which has been introduced by the user in a syntactically correct way.
//  The acceptable range is specified during the call;
//   eventually, the error message is returned
extern pair<QString,string>
revalidate(const QString& defText,const double& def,
	   const double& min,const double& max,
	   const string& valueName,const QString& value);
// In this function we double commata, to produce strings
//  compatible to feynmp macros
extern string taint(const string& metaPostString);

// Class definitions

class vertex
{
  pair<int,int> pos;
  string type;
  float startingAngle;
  float size;
  float border;
  string fill;
  string foreColor;
  string backColor;
  int depth;
  bool isSticky;
public:
  vertex(pair<int,int> iPos,string iType="circle",
	 float iStartingAngle=0,float iSize=1.5,float iBorder=1,
	 string iFill="",string iForeColor="black",string iBackColor="black",
	 int iDepth=50,bool iIsSticky=false):
    pos(iPos), type(iType), startingAngle(iStartingAngle), size(iSize),
    border(iBorder), fill(iFill), foreColor(iForeColor), backColor(iBackColor),
    depth(iDepth), isSticky(iIsSticky) { }

  vertex(QDomElement& node):
    pos(make_pair(node.attribute("x").toUInt(),node.attribute("y").toUInt())),
    type(node.attribute("type","circle").ascii()),
    startingAngle(node.attribute("angle","0").toFloat()),
    size(node.attribute("size","1.5").toFloat()),
    border(node.attribute("border","1").toFloat()),
    fill(node.attribute("fill","").ascii()),
    foreColor(node.attribute("foreColor","black").ascii()),
    backColor(node.attribute("backColor","black").ascii()),
    depth(node.attribute("depth","50").toInt()),
    isSticky(bool(node.attribute("sticky","0").toInt())) { }

  ~vertex() { }

  string toMetapost() const {
    ostringstream mp;
    if (this->type!="")
      mp << "FeynCADVertex(\"decoration.shape=" << this->type
	 << ",decoration.size=" << this->size << "thin,decoration.border="
	 << this->border << "thin,decoration.fill="
	 << (this->fill==""?"0":this->fill) << ",decoration.angle="
	 << this->startingAngle << ",foreground=" << taint(this->foreColor)
	 << ",background=" << taint(this->backColor) << "\",("
	 << this->pos.first << UNIT << "," << this->pos.second << UNIT
	 << "));" << endl;
    return mp.str();
  }

  void toXml(QDomDocument& document,QDomElement& rootNode) const {
    QDomElement vertexTag=document.createElement("vertex");
    vertexTag.setAttribute("x",QString::number(this->pos.first));
    vertexTag.setAttribute("y",QString::number(this->pos.second));
    vertexTag.setAttribute("type",QString(this->type.c_str()));
    vertexTag.setAttribute("angle",QString::number(this->startingAngle));
    vertexTag.setAttribute("size",QString::number(this->size));
    vertexTag.setAttribute("border",QString::number(this->border));
    vertexTag.setAttribute("fill",QString(this->fill.c_str()));
    vertexTag.setAttribute("foreColor",QString(this->foreColor.c_str()));
    vertexTag.setAttribute("backColor",QString(this->backColor.c_str()));
    vertexTag.setAttribute("depth",QString::number(this->depth));
    vertexTag.setAttribute("sticky",QString::number(int(this->isSticky)));
    rootNode.appendChild(vertexTag);
    return;
  }

  friend inline bool operator==(const vertex& a,const vertex& b);
  friend inline bool operator<(const vertex& a,const vertex& b);
  friend struct depthLess;
  struct depthLess {
    inline bool operator()(const vertex& a,const vertex& b) {
      return (a.depth<b.depth||a.depth==b.depth&&
	      (a.pos.first<b.pos.first||
	       (a.pos.first==b.pos.first&&a.pos.second<b.pos.second)));
    }
  };
  
  friend class QFeynCADVertexEdit;
  friend class QFeynCADTransformEdit;
  friend class QFeynCADEditor;
  friend class QFeynCADCombined;
};

class line
{
  pair<int,int> vOne, vTwo;
  string type;
  float angleOne, angleTwo;
  float tension; // It is the distance in the case of tadpoles
  float size;
  string color;
  float rubout;
  int depth;
  // A trick for alpha selection
  bool selOne, selTwo;
public:
  // La linea
  line(pair<int,int> iVOne,pair<int,int> iVTwo,string iType="plain",
       float iAngleOne=-361,float iAngleTwo=-361,float iTension=1,
       float iSize=1,string iColor="black",float iRubout=1,int iDepth=75):
    vOne(iVOne), vTwo(iVTwo), type(iType), angleOne(iAngleOne),
    angleTwo(iAngleTwo), tension(iTension), size(iSize), color(iColor),
    rubout(iRubout), depth(iDepth), selOne(false), selTwo(false) { }
  // Il tadpolo
  line(pair<int,int> iVertex,string iType="plain",
       float iAngleOne=90,float iAngleTwo=90,float iTension=4,
       float iSize=1,string iColor="black",float iRubout=1,int iDepth=75):
    vOne(iVertex), vTwo(iVertex), type(iType), angleOne(iAngleOne),
    angleTwo(iAngleTwo), tension(iTension), size(iSize), color(iColor),
    rubout(iRubout), depth(iDepth), selOne(false), selTwo(false) { }

  line(QDomElement& node):
    vOne(make_pair(node.attribute("xOne").toUInt(),
		   node.attribute("yOne").toUInt())),
    vTwo(make_pair(node.attribute("xTwo").toUInt(),
		   node.attribute("yTwo").toUInt())),
    type(node.attribute("type","plain").ascii()),
    angleOne(node.attribute("angleOne","-361").toFloat()),
    angleTwo(node.attribute("angleTwo","-361").toFloat()),
    tension(node.attribute("tension","1").toFloat()),
    size(node.attribute("size","1").toFloat()),
    color(node.attribute("color","black").ascii()),
    rubout(node.attribute("rubout","1").toFloat()),
    depth(node.attribute("depth","75").toInt()),
    selOne(false), selTwo(false) {}

  ~line() { }

  string toMetapost() const {
    ostringstream mp;
    // Case of "accidental tadpoles"
    if (this->vOne==this->vTwo&&!this->tension)
      return mp.str();
    if (this->vOne==this->vTwo) {
      mp << "FeynCADLine(\"" << this->type << ",width=" << this->size << "thin"
	 << ",foreground=" << taint(this->color) << ",rubout=" << this->rubout
	 << "\",(" << this->vOne.first << UNIT << "," << this->vOne.second
	 << UNIT << "){dir (" << this->angleOne << "-(" << this->angleTwo
	 << "/2))}..(" << this->vOne.first << UNIT << "," << this->vOne.second
	 << UNIT << ")+((" << this->tension << UNIT << ",0) rotated ("
	 << this->angleOne << ")){dir (" << (this->angleTwo<0?"-":"")
	 << "90+(" << this->angleOne << "))}..{dir (180+(" << this->angleOne
	 << ")+(" << this->angleTwo << "/2))}";
      if (this->angleTwo!=180&&this->angleTwo!=-180)
	mp << "(" << this->vOne.first << UNIT << "," << this->vOne.second
	   << UNIT << ")";
      else
	mp << "cycle";
      mp << ");" << endl;
    } else {
      mp << "FeynCADLine(\"" << this->type << ",width=" << this->size << "thin"
	 << ",foreground=" << taint(this->color) << ",rubout=" << this->rubout
	 << "\",(" << this->vOne.first << UNIT << "," << this->vOne.second
	 << UNIT << ")";
      if (this->angleOne!=-361)
	mp << "{dir " << this->angleOne << "}";
      mp << "..";
      mp << "tension " << this->tension << " and " << this->tension << "..";
      if (this->angleTwo!=-361)
	mp << "{dir " << this->angleTwo << "}";
      mp << "(" << this->vTwo.first << UNIT << "," << this->vTwo.second
	 << UNIT << "));" << endl;
    }
    return mp.str();
  }

  void toXml(QDomDocument& document,QDomElement& rootNode) const {
    QDomElement lineTag=document.createElement("line");
    lineTag.setAttribute("xOne",QString::number(this->vOne.first));
    lineTag.setAttribute("yOne",QString::number(this->vOne.second));
    lineTag.setAttribute("xTwo",QString::number(this->vTwo.first));
    lineTag.setAttribute("yTwo",QString::number(this->vTwo.second));
    lineTag.setAttribute("type",QString(this->type.c_str()));
    lineTag.setAttribute("angleOne",QString::number(this->angleOne));
    lineTag.setAttribute("angleTwo",QString::number(this->angleTwo));
    lineTag.setAttribute("tension",QString::number(this->tension));
    lineTag.setAttribute("size",QString::number(this->size));
    lineTag.setAttribute("color",QString(this->color.c_str()));
    lineTag.setAttribute("rubout",QString::number(this->rubout));
    lineTag.setAttribute("depth",QString::number(this->depth));
    rootNode.appendChild(lineTag);
    return;
  }

  friend inline bool operator==(const line& a,const line& b);
  friend inline bool operator<(const line& a,const line& b);
  friend struct depthLess;
  struct depthLess {
    inline bool operator()(const line& a,const line& b) {
      return (a.depth<b.depth||a.depth==b.depth&&
	      (a.vOne<b.vOne||(a.vOne==b.vOne&&a.vTwo<b.vTwo)));
    }
  };
  
  friend class QFeynCADLineEdit;
  friend class QFeynCADTadpoleEdit;
  friend class QFeynCADTransformEdit;
  friend class QFeynCADEditor;
  friend class QFeynCADCombined;
};

class tag
{
private:
  pair<int,int> pos;
  string text;
  string font;
  string color;
  float scale;
  float startingAngle;
  string controlPoint;
  int depth;
public:
  tag(pair<int,int> iPos,string iText="The Text",
      string iFont="",string iColor="black",float iScale=1,
      float iStartingAngle=0,string iControlPoint="cc",int iDepth=25):
    pos(iPos), text(iText), font(iFont), color(iColor), scale(iScale),
    startingAngle(iStartingAngle), controlPoint(iControlPoint), depth(iDepth)
  { }

  tag(QDomElement& node):
    pos(make_pair(node.attribute("x").toUInt(),node.attribute("y").toUInt())),
    text(node.attribute("text","The Text").ascii()),
    font(node.attribute("font","").ascii()),
    color(node.attribute("color","black").ascii()),
    scale(node.attribute("scale","1").toFloat()),
    startingAngle(node.attribute("angle","0").toFloat()),
    // The default for the tag is to be centered
    controlPoint(node.attribute("point","cc").ascii()),
    depth(node.attribute("depth","25").toInt()) { }

  ~tag() { }

  string toMetapost() const {
    ostringstream mp;
    mp << "draw thelabel";
    if (this->controlPoint=="tl") mp << ".lrt(";
    else if (this->controlPoint=="tc") mp << ".bot(";
    else if (this->controlPoint=="tr") mp << ".llft(";
    else if (this->controlPoint=="cl") mp << ".rt(";
    else if (this->controlPoint=="cr") mp << ".lft(";
    else if (this->controlPoint=="bl") mp << ".urt(";
    else if (this->controlPoint=="bc") mp << ".top(";
    else if (this->controlPoint=="br") mp << ".ulft(";
    // Il caso "cc" serve da default
    else mp << "(";
    if (this->font=="")
      mp << "btex " << this->text << " etex";
    else
      mp << "\"" << this->text << "\" infont \"" << this->font << "\"";
    mp << ",(0,0)) scaled " << this->scale << " rotated "
       << this->startingAngle << " shifted (" << this->pos.first << UNIT << ","
       << this->pos.second << UNIT << ") withcolor " << taint(this->color)
       << ";" << endl;
    return mp.str();
  }

  void toXml(QDomDocument& document,QDomElement& rootNode) const {
    QDomElement tagTag=document.createElement("tag");
    tagTag.setAttribute("x",QString::number(this->pos.first));
    tagTag.setAttribute("y",QString::number(this->pos.second));
    tagTag.setAttribute("text",QString(this->text.c_str()));
    tagTag.setAttribute("font",QString(this->font.c_str()));
    tagTag.setAttribute("color",QString(this->color.c_str()));
    tagTag.setAttribute("scale",QString::number(this->scale));
    tagTag.setAttribute("angle",QString::number(this->startingAngle));
    tagTag.setAttribute("point",QString(this->controlPoint.c_str()));
    tagTag.setAttribute("depth",QString::number(this->depth));
    rootNode.appendChild(tagTag);
    return;
  }

  friend inline bool operator==(const tag& a,const tag& b);
  friend inline bool operator<(const tag& a,const tag& b);
  friend struct depthLess;
  struct depthLess {
    inline bool operator()(const tag& a,const tag& b) {
      return (a.depth<b.depth||a.depth==b.depth&&
	      (a.pos.first<b.pos.first||
	       (a.pos.first==b.pos.first&&a.pos.second<b.pos.second)));
    }
  };
  
  friend class QFeynCADTagEdit;
  friend class QFeynCADTransformEdit;
  friend class QFeynCADEditor;
  friend class QFeynCADCombined;
};

inline bool operator==(const vertex& a,const vertex& b)
{ return (a.pos.first==b.pos.second&&a.pos.first==b.pos.second); }
inline bool operator<(const vertex& a,const vertex& b) {
  return (a.pos.first<b.pos.first||
	  (a.pos.first==b.pos.first&&a.pos.second<b.pos.second));
}

inline bool operator==(const line& a,const line& b)
{ return (a.vOne==b.vOne&&a.vTwo==b.vTwo); }
inline bool operator<(const line& a,const line& b)
{ return (a.vOne<b.vOne||(a.vOne==b.vOne&&a.vTwo<b.vTwo)); }

inline bool operator==(const tag& a,const tag& b)
{ return (a.pos.first==b.pos.second&&a.pos.first==b.pos.second); }
inline bool operator<(const tag& a,const tag& b) {
  return (a.pos.first<b.pos.first||
	  (a.pos.first==b.pos.first&&a.pos.second<b.pos.second));
}

#endif // FEYNCAD_STRUCTURES_GUARD
